@@ -1,3 +1,7 @@
+#ifndef _CONFIG_H_
+#define _CONFIG_H_
+@TOP@
+
 /* Define this if you're running x86 architecture */
 #undef __i386__
 
@@ -39,8 +43,23 @@
 /* Define this if you have libXinerama installed */
 #undef HAVE_XINERAMA
 
+/* Define this if you have libXxf86vm installed */
+#undef HAVE_XF86VIDMODE
+
 /* Define this if you have DPMS */
 #undef HAVE_DPMS
 
 /* Define this to add mrl browser into xitk lib */
 #undef NEED_MRLBROWSER
+
+/* Where scripts are */
+#undef XINE_SCRIPTDIR
+
+/* Path where catalog files will be. */
+#undef XITK_LOCALE
+
+/* Path where docs files will be. */
+#undef XINE_DOCDIR
+
+@BOTTOM@
+#endif
