@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: control.h,v 1.4 2001/06/21 22:13:41 f1rmb Exp $
+ * $Id: control.h,v 1.5 2001/10/21 00:28:53 f1rmb Exp $
  *
  */
 
@@ -25,10 +25,11 @@
 #define CONTROL_H
 
 void control_panel(void);
-void control_exit(widget_t *, void *);
+void control_change_skins(void);
+void control_exit(xitk_widget_t *, void *);
 int control_is_visible(void);
 int control_is_running(void);
-void control_toggle_panel_visibility(widget_t *, void *);
+void control_toggle_panel_visibility(xitk_widget_t *, void *);
 void control_raise_window(void);
 
 #endif
