@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: mrl_browser.h,v 1.2 2001/07/14 00:16:43 f1rmb Exp $
+ * $Id: mrl_browser.h,v 1.5 2001/11/18 15:58:42 f1rmb Exp $
  *
  */
 
@@ -28,15 +28,16 @@
 #include "config.h"
 #endif
 
+#include <xine.h>
+
 #include "xitk.h"
 
-#include "xine.h"
+typedef void (*select_cb_t) (xitk_widget_t *, void *);
 
-typedef void (*select_cb_t) (widget_t *, void *);
-
-void mrl_browser(xitk_mrl_callback_t add_cb,
+void mrl_browser(xitk_mrl_callback_t add_cb, xitk_mrl_callback_t add_and_play_cb,
 		 select_cb_t sel_cb, xitk_dnd_callback_t dnd_cb);
-void open_mrlbrowser(widget_t *w, void *data);
+void mrl_browser_change_skins(void);
+void open_mrlbrowser(xitk_widget_t *w, void *data);
 void destroy_mrl_browser(void);
 int mrl_browser_is_running(void);
 int mrl_browser_is_visible(void);
