@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: playlist.h,v 1.5 2001/07/10 21:04:27 f1rmb Exp $
+ * $Id: playlist.h,v 1.7 2001/10/21 00:28:54 f1rmb Exp $
  *
  */
 
@@ -25,11 +25,13 @@
 #define PLAYLIST_H
 
 void playlist_editor(void);
-void pl_exit(widget_t *, void *);
+void playlist_change_skins(void);
+void pl_exit(xitk_widget_t *, void *);
 int pl_is_running(void);
-void pl_toggle_visibility(widget_t *, void *);
+int pl_is_visible(void);
+void pl_toggle_visibility(xitk_widget_t *, void *);
 void pl_raise_window(void);
-void pl_scan_input(widget_t *, void *);
+void pl_scan_input(xitk_widget_t *, void *);
 void pl_update_playlist(void);
 
 #endif
