@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: videowin.c,v 1.30 2001/09/13 09:00:53 siggi Exp $
+ * $Id: videowin.c,v 1.61 2002/01/03 22:45:01 jcdutton Exp $
  *
  * video window handling functions
 */
@@ -32,28 +32,30 @@
 #ifdef HAVE_XINERAMA
 #include <X11/extensions/Xinerama.h>
 #endif
+#ifdef HAVE_XF86VIDMODE
+#include <X11/extensions/xf86vmode.h>
+#endif
 
 #include <xine.h>
 #include <xine/video_out_x11.h>
-
-#include "xine.h"
+#include <xine/xineutils.h>
 
 #include "xitk.h"
 
-#include "utils.h"
 #include "Imlib-light/Imlib.h"
 
 #include "event.h"
 #include "videowin.h"
 #include "panel.h"
+#include "actions.h"
 
 extern gGui_t *gGui;
 
 /* Video window private structure */
 typedef struct {
-  XVisualInfo    vinfo;
   Cursor         cursor[2];       /* Cursor pointers                       */
   int            cursor_visible;
+  Visual	*visual;	  /* Visual for video window               */
   Colormap	 colormap;	  /* Colormap for video window		   */
   XClassHint    *xclasshint;
   GC             gc;
@@ -64,16 +66,29 @@ typedef struct {
 				   * switch to fullscreen mode             */
   int            fullscreen_width;
   int            fullscreen_height;
+#ifdef HAVE_XINERAMA
+  XineramaScreenInfo *xinerama;   /* pointer to xinerama struct, or NULL */
+  int            xinerama_cnt;    /* number of screens in Xinerama */
+#endif
+  int            xwin;            /* current X location */
+  int            ywin;            /* current Y location */
+  int            desktopWidth;    /* desktop width */
+  int            desktopHeight;   /* desktop height */
   int            completion_type;
   int            depth;
   int            show;
   XWMHints      *wm_hint;
 
-  widgetkey_t    widget_key;
-  widgetkey_t    old_widget_key;
+  xitk_register_key_t    widget_key;
+  xitk_register_key_t    old_widget_key;
 
   int            completion_event;
 
+#ifdef HAVE_XF86VIDMODE
+  /* XF86VidMode Extension stuff */
+  XF86VidModeModeInfo** XF86_modelines;
+  int                   XF86_modelines_count;
+#endif
 } gVw_t;
 
 static gVw_t    *gVw;
@@ -88,8 +103,9 @@ static void video_window_handle_event (XEvent *event, void *data);
  * Will called by toolkit on every move/resize event.
  */
 void video_window_change_sizepos(int x, int y, int w, int h) {
-  
-  //  printf("video window change: %d %d %d %d\n", x, y, w, h);
+  gVw->xwin = x;
+  gVw->ywin = y;
+  /*  printf("video window change: %d %d %d %d\n", x, y, w, h); */
 }
 
 /*
@@ -113,7 +129,7 @@ void video_window_draw_logo(void) {
       
       tmp = (wwin / 100) * 86;
       ratio = (tmp < gGui->video_window_logo_pixmap.width && tmp >= 1) ? 
-	(ratio = (double)tmp / (double)gGui->video_window_logo_pixmap.width):1;
+	((double)tmp / (double)gGui->video_window_logo_pixmap.width):1;
     }
     
     resized_image = Imlib_clone_image(gGui->imlib_data, 
@@ -156,22 +172,45 @@ void video_window_hide_logo(void) {
      * Remove the logo by clearing the window.
      */
     XClearWindow (gGui->display, gGui->video_window);
+
+    /* Force ConfigureNotify event */
+    /* 
+     *   no longer needed, video_out_xv repaints it's colorkey now on
+     * GUI_DATA_EX_LOGO_VISIBILITY 
+     *
+    {
+      Window        rootwin;
+      int           xwin, ywin;
+      unsigned int  wwin, hwin, bwin, dwin;
+      
+      if(XGetGeometry(gGui->display, gGui->video_window, &rootwin, 
+		      &xwin, &ywin, &wwin, &hwin, &bwin, &dwin) != BadDrawable) {
+	
+	XMoveResizeWindow (gGui->display, gGui->video_window, xwin, ywin, wwin-1, hwin-1);
+	XMoveResizeWindow (gGui->display, gGui->video_window, xwin, ywin, wwin, hwin);
+      }
+    }
+    */
+    
+    gGui->vo_driver->gui_data_exchange(gGui->vo_driver, GUI_DATA_EX_LOGO_VISIBILITY, (int *)0);
 #endif
     XUnlockDisplay (gGui->display);
   }
-  /*
+
+/* 2001-10-07 ehasenle: activated this code for dxr3 overlay */
     gGui->vo_driver->gui_data_exchange (gGui->vo_driver,
     GUI_DATA_EX_DRAWABLE_CHANGED, 
     (void*)gGui->video_window);
-  */
 }
 
 /*
  * Show the logo in video output window.
  */
 void video_window_show_logo(void) {
-
+   
   if(video_window_is_visible()) {
+    gGui->vo_driver->gui_data_exchange (gGui->vo_driver, GUI_DATA_EX_LOGO_VISIBILITY, (int *)1);
+     
     XLockDisplay (gGui->display);
     XClearWindow (gGui->display, gGui->video_window); 
     video_window_draw_logo();
@@ -188,12 +227,15 @@ void video_window_set_fullscreen (int req_fullscreen) {
 
   gVw->fullscreen_req = req_fullscreen;
 
-  video_window_adapt_size (gVw->video_width, gVw->video_height, 
+  video_window_adapt_size (NULL, gVw->video_width, gVw->video_height, 
 			   &area.x, &area.y, &area.w, &area.h);
 
   gGui->vo_driver->gui_data_exchange (gGui->vo_driver, 
 				      GUI_DATA_EX_DEST_POS_SIZE_CHANGED, 
 				      &area);
+
+  /* Tell the video driver if we are in fullscreen mode or not */
+  gGui->vo_driver->gui_data_exchange(gGui->vo_driver, GUI_DATA_EX_FULLSCREEN, (int *)gVw->fullscreen_mode);
 }
 
 /*
@@ -206,13 +248,30 @@ int video_window_is_fullscreen (void) {
 /*
  *
  */
-void video_window_calc_dest_size (int video_width, int video_height,
+void video_window_calc_dest_size (void *this,
+				  int video_width, int video_height,
 				  int *dest_width, int *dest_height)  {
 
   if (gVw->fullscreen_mode) {
 
+#ifdef HAVE_XINERAMA
+    if (gVw->xinerama) {
+        int i;
+        for (i = 0; i < gVw->xinerama_cnt; i++) {
+            if (gVw->xinerama[i].screen_number == XScreenNumberOfScreen(XDefaultScreenOfDisplay(gGui->display))) {
+                *dest_width = gVw->xinerama[i].width;
+                *dest_height = gVw->xinerama[i].height;
+                break;
+            }
+        }
+    } else {
+        *dest_width  = gVw->fullscreen_width;
+        *dest_height = gVw->fullscreen_height;
+    }
+#else
     *dest_width  = gVw->fullscreen_width;
     *dest_height = gVw->fullscreen_height;
+#endif
     
   } else {
 
@@ -225,7 +284,8 @@ void video_window_calc_dest_size (int video_width, int video_height,
 /*
  *
  */
-void video_window_adapt_size (int video_width, int video_height, 
+void video_window_adapt_size (void *this,
+			      int video_width, int video_height, 
 			      int *dest_x, int *dest_y,
 			      int *dest_width, int *dest_height) {
 
@@ -242,6 +302,15 @@ void video_window_adapt_size (int video_width, int video_height,
   Window                old_video_window = None;
   long data[1];
 
+/*  printf("window_adapt:vw=%d, vh=%d, dx=%d, dy=%d, dw=%d, dh=%d\n",
+ *           video_width,
+ *           video_height,
+ *           dest_x,
+ *           dest_y,
+ *           dest_width,
+ *           dest_height); 
+ */
+
   XLockDisplay (gGui->display);
 
   gVw->video_width = video_width;
@@ -249,19 +318,138 @@ void video_window_adapt_size (int video_width, int video_height,
   *dest_x = 0;
   *dest_y = 0;
 
+#ifdef HAVE_XF86VIDMODE
+  /* XF86VidMode Extension
+   * In case a fullscreen request is received or if already in fullscreen, the
+   * appropriate modeline will be looked up and used.
+   */
+  if(gVw->fullscreen_req || gVw->fullscreen_mode) {
+    int search = 0;
+    
+    /* skipping first entry because it is the current modeline */
+    for(search = 1; search < gVw->XF86_modelines_count; search++) {
+       if(gVw->XF86_modelines[search]->hdisplay >= gVw->video_width)
+	 break;
+    }
+
+    /*
+     * in case we have a request for a resolution higher than any available
+     * ones we take the highest currently available.
+     */
+    if(gVw->fullscreen_mode && search >= gVw->XF86_modelines_count)
+       search = 0;
+       
+    /* just switching to a different modeline if necessary */
+    if(!(gVw->XF86_modelines_count <= 1) && !(search >= gVw->XF86_modelines_count)) {
+       if(XF86VidModeSwitchToMode(gGui->display, XDefaultScreen(gGui->display), gVw->XF86_modelines[search])) {
+	  XF86VidModeSetViewPort(gGui->display, XDefaultScreen(gGui->display), 0, 0);
+          
+	  gGui->XF86VidMode_fullscreen = 1;
+	  
+	  gVw->fullscreen_width  = gVw->XF86_modelines[search]->hdisplay;
+	  gVw->fullscreen_height = gVw->XF86_modelines[search]->vdisplay;
+	  
+	  /*
+	   *just in case the mouse pointer is off the visible area, move it
+	   * to the middle of the video window
+	   */
+	  XWarpPointer(gGui->display, None, gGui->video_window, 0, 0, 0, 0, gVw->fullscreen_width/2, gVw->fullscreen_height/2);
+	  
+	  /*
+	   * if this is true, we are back at the original resolution, so there
+	   * is no need to further worry about anything.
+	   */
+	  if(gVw->fullscreen_mode && search == 0)
+	    gGui->XF86VidMode_fullscreen = 0;
+       } else {
+	  xine_error("XF86VidMode Extension: modeline switching failed.\n");
+       }
+    }
+  }
+#endif
+
+#ifdef HAVE_XINERAMA
+    if (gVw->xinerama) {
+        int i;
+        int knowLocation = 0;
+
+        /* someday this could also use the centre of the window as the
+         * test point I guess.  Right now it's the upper-left.
+         */
+        if (gGui->video_window != None) {
+            if (gVw->xwin >= 0 && gVw->ywin >= 0 &&
+              gVw->xwin < gVw->desktopWidth && gVw->ywin < gVw->desktopHeight) {
+                knowLocation = 1;
+            }
+        }
+
+        for (i = 0; i < gVw->xinerama_cnt; i++) {
+            if (
+                (knowLocation == 1 &&
+                 gVw->xwin >= gVw->xinerama[i].x_org &&
+                 gVw->ywin >= gVw->xinerama[i].y_org &&
+                 gVw->xwin <= gVw->xinerama[i].x_org+gVw->xinerama[i].width &&
+                 gVw->ywin <= gVw->xinerama[i].y_org+gVw->xinerama[i].height) ||
+                (knowLocation == 0 &&
+                 gVw->xinerama[i].screen_number == 
+                   XScreenNumberOfScreen(XDefaultScreenOfDisplay(gGui->display)))) {
+                hint.x = gVw->xinerama[i].x_org;
+                hint.y = gVw->xinerama[i].y_org;
+                if (gVw->fullscreen_req) {
+                    hint.width  = gVw->xinerama[i].width;
+                    hint.height = gVw->xinerama[i].height;
+                    gVw->fullscreen_width = hint.width;
+                    gVw->fullscreen_height = hint.height;
+                } else {
+                    hint.width  = gVw->video_width;
+                    hint.height = gVw->video_height;
+                }
+                *dest_width = hint.width;
+                *dest_height = hint.height;
+                break;
+            }
+        }
+    } else {
+        hint.x = 0;
+        hint.y = 0;
+        if (gVw->fullscreen_req) {
+            hint.width  = gVw->fullscreen_width;
+            hint.height = gVw->fullscreen_height;
+        } else {
+            hint.width  = gVw->video_width;
+            hint.height = gVw->video_height;
+        }
+        *dest_width  = hint.width;
+        *dest_height = hint.height;
+    }
+#else
+    hint.x = 0;
+    hint.y = 0;   /* for now -- could change later */
+#endif
+
   if (gVw->fullscreen_req) {
 
+#ifndef HAVE_XINERAMA
     *dest_width  = gVw->fullscreen_width;
     *dest_height = gVw->fullscreen_height;
+#endif
 
     if (gGui->video_window) {
 
       if (gVw->fullscreen_mode) {
+#ifdef HAVE_XF86VIDMODE
+	/*
+	 * resizing the video window may be necessary if the modeline has
+	 * just been switched
+	 */
+	XResizeWindow (gGui->display, gGui->video_window,
+		       gVw->fullscreen_width, gVw->fullscreen_height);
+#endif
 	XUnlockDisplay (gGui->display);
 	return;
       }
 
-      widget_unregister_event_handler(&gVw->old_widget_key);
+      xitk_unregister_event_handler(&gVw->old_widget_key);
       old_video_window = gGui->video_window;
     }
 
@@ -272,24 +460,15 @@ void video_window_adapt_size (int video_width, int video_height,
      */
 
     attr.background_pixel  = gGui->black.pixel;
-    /*
-     * XXX:multivis
-     * To avoid BadMatch errors on XCreateWindow: 
-     * If the parent and the new window have different depths, we must
-     * supply either a BorderPixmap or a BorderPixel.
-     * If the parent and the new window use different visuals, we must
-     * supply a Colormap
-     */
     attr.border_pixel      = gGui->black.pixel;
     attr.colormap	   = gVw->colormap;
 
     gGui->video_window = 
-      XCreateWindow (gGui->display, 
-		     RootWindow (gGui->display, DefaultScreen(gGui->display)), 
-		     0, 0, gVw->fullscreen_width, 
+      XCreateWindow (gGui->display, gGui->imlib_data->x.root, 
+		     hint.x, hint.y, gVw->fullscreen_width, 
 		     gVw->fullscreen_height, 
 		     0, gVw->depth, CopyFromParent, 
-		     gVw->vinfo.visual,
+		     gVw->visual,
 		     CWBackPixel  | CWBorderPixel | CWColormap, &attr);
     
     if(gGui->vo_driver)
@@ -300,10 +479,12 @@ void video_window_adapt_size (int video_width, int video_height,
     if (gVw->xclasshint != NULL)
       XSetClassHint(gGui->display, gGui->video_window, gVw->xclasshint);
 
+#ifndef HAVE_XINERAMA
     hint.x = 0;
     hint.y = 0;
     hint.width  = gVw->fullscreen_width;
     hint.height = gVw->fullscreen_height;
+#endif
     hint.win_gravity = StaticGravity;
     hint.flags  = PPosition | PSize | PWinGravity;
     
@@ -311,10 +492,25 @@ void video_window_adapt_size (int video_width, int video_height,
  			   window_title, window_title, None, NULL, 0, 0);
     
     XSetWMNormalHints (gGui->display, gGui->video_window, &hint);
-    
-    
+        
     XSetWMHints(gGui->display, gGui->video_window, gVw->wm_hint);
 
+    
+    /*
+     * layer above most other things, like gnome panel
+     * WIN_LAYER_ABOVE_DOCK  = 10
+     *
+     */
+    if(gGui->layer_above) {
+      if( XA_WIN_LAYER == None )
+	XA_WIN_LAYER = XInternAtom(gGui->display, "_WIN_LAYER", False);
+      
+      data[0] = 10;
+      XChangeProperty(gGui->display, gGui->video_window, XA_WIN_LAYER,
+		      XA_CARDINAL, 32, PropModeReplace, (unsigned char *)data,
+		      1);
+    }
+
     /*
      * wm, no borders please
      */
@@ -326,26 +522,12 @@ void video_window_adapt_size (int video_width, int video_height,
 		    PropModeReplace, (unsigned char *) &mwmhints,
 		    PROP_MWM_HINTS_ELEMENTS);
 
-    /*
-     * layer above most other things, like gnome panel
-     * WIN_LAYER_ABOVE_DOCK  = 10
-     *
-     */
-
-    if( XA_WIN_LAYER == None )
-      XA_WIN_LAYER = XInternAtom(gGui->display, "_WIN_LAYER", False);
-    
-    data[0] = 10;
-    XChangeProperty(gGui->display, gGui->video_window, XA_WIN_LAYER,
-		    XA_CARDINAL, 32, PropModeReplace, (unsigned char *)data,
-		    1);
-    
-
     XSetTransientForHint(gGui->display, gGui->video_window, None);
     XRaiseWindow(gGui->display, gGui->video_window);
 
   } else {
 
+#ifndef HAVE_XINERAMA
     *dest_width  = gVw->video_width;
     *dest_height = gVw->video_height;
 
@@ -353,12 +535,29 @@ void video_window_adapt_size (int video_width, int video_height,
     hint.y = 0;
     hint.width  = gVw->video_width;
     hint.height = gVw->video_height;
+#endif
     hint.flags  = PPosition | PSize;
 
     if (gGui->video_window) {
 
       if (gVw->fullscreen_mode) {
-	widget_unregister_event_handler(&gVw->old_widget_key);
+#ifdef HAVE_XF86VIDMODE
+	/*
+	 * toggling from fullscreen to window mode - time to switch back to
+	 * the original modeline
+	 */
+	if(gVw->XF86_modelines_count > 1) {
+	   XF86VidModeSwitchToMode(gGui->display, XDefaultScreen(gGui->display), gVw->XF86_modelines[0]);
+	   XF86VidModeSetViewPort(gGui->display, XDefaultScreen(gGui->display), 0, 0);
+
+	   gGui->XF86VidMode_fullscreen = 0;
+       
+	   gVw->fullscreen_width  = gVw->XF86_modelines[0]->hdisplay;
+	   gVw->fullscreen_height = gVw->XF86_modelines[0]->vdisplay;
+	}
+#endif
+
+	xitk_unregister_event_handler(&gVw->old_widget_key);
 	old_video_window = gGui->video_window;
       }
       else {
@@ -379,21 +578,13 @@ void video_window_adapt_size (int video_width, int video_height,
     gVw->fullscreen_mode = 0;
 
     attr.background_pixel  = gGui->black.pixel;
-    /*
-     * XXX:multivis
-     * To avoid BadMatch errors on XCreateWindow: 
-     * If the parent and the new window have different depths, we must
-     * supply either a BorderPixmap or a BorderPixel.
-     * If the parent and the new window use different visuals, we must
-     * supply a Colormap
-     */
     attr.border_pixel      = gGui->black.pixel;
     attr.colormap	   = gVw->colormap;
-
+    
     gGui->video_window = 
-      XCreateWindow(gGui->display, RootWindow(gGui->display, gGui->screen),
+      XCreateWindow(gGui->display, gGui->imlib_data->x.root,
 		    hint.x, hint.y, hint.width, hint.height, 4, 
-		    gVw->depth, CopyFromParent, gVw->vinfo.visual,
+		    gVw->depth, CopyFromParent, gVw->visual,
 		    CWBackPixel | CWBorderPixel | CWColormap, &attr);
     
     if(gGui->vo_driver)
@@ -422,7 +613,8 @@ void video_window_adapt_size (int video_width, int video_height,
   if (wm_hint != NULL) {
     wm_hint->input = True;
     wm_hint->initial_state = NormalState;
-    wm_hint->flags = InputHint | StateHint;
+    wm_hint->icon_pixmap = gGui->icon;
+    wm_hint->flags = InputHint | StateHint | IconPixmapHint;
     XSetWMHints(gGui->display, gGui->video_window, wm_hint);
     XFree(wm_hint);
   }
@@ -442,14 +634,18 @@ void video_window_adapt_size (int video_width, int video_height,
   } while (xev.type != MapNotify || xev.xmap.event != gGui->video_window);
 
   XFlush(gGui->display);
-  
+
   if (gVw->gc != None) XFreeGC(gGui->display, gVw->gc);
   gVw->gc = XCreateGC(gGui->display, gGui->video_window, 0L, &xgcv);
-
+  
   if (gVw->fullscreen_mode) {
     XSetInputFocus (gGui->display, 
 		    gGui->video_window, RevertToNone, CurrentTime);
+#ifdef HAVE_XINERAMA
+    XMoveWindow (gGui->display, gGui->video_window, hint.x, hint.y);
+#else
     XMoveWindow (gGui->display, gGui->video_window, 0, 0);
+#endif
   }
 
   /* Update WM_TRANSIENT_FOR hints on other windows for the new video_window */
@@ -462,15 +658,18 @@ void video_window_adapt_size (int video_width, int video_height,
   /* The old window should be destroyed now */
   if(old_video_window != None) {
     XDestroyWindow(gGui->display, old_video_window);
+     
+    if(gGui->cursor_grabbed)
+       XGrabPointer(gGui->display, gGui->video_window, 1, None, GrabModeAsync, GrabModeAsync, gGui->video_window, None, CurrentTime);
   }
 
   gVw->old_widget_key = gVw->widget_key;
-  gVw->widget_key = widget_register_event_handler("video_window", 
-						  gGui->video_window, 
-						  video_window_handle_event,
-						  video_window_change_sizepos,
-						  gui_dndcallback,
-						  NULL, NULL);
+  gVw->widget_key = xitk_register_event_handler("video_window", 
+						gGui->video_window, 
+						video_window_handle_event,
+						video_window_change_sizepos,
+						gui_dndcallback,
+						NULL, NULL);
   
   XUnlockDisplay (gGui->display);
 }
@@ -503,24 +702,31 @@ void video_window_set_visibility(int show_window) {
 
   static Atom  XA_WIN_LAYER = None;
   long         data[1];
-
+  
+  gGui->vo_driver->gui_data_exchange (gGui->vo_driver, 
+				      GUI_DATA_EX_VIDEOWIN_VISIBLE, 
+				      (int *)show_window);
+  
   gVw->show = show_window;
 
   if (gVw->show == 1) {
     XLockDisplay (gGui->display);
+
     /*
      * layer above most other things, like gnome panel
      * WIN_LAYER_ABOVE_DOCK  = 10
      *
      */
-
-    if( XA_WIN_LAYER == None )
-      XA_WIN_LAYER = XInternAtom(gGui->display, "_WIN_LAYER", False);
+    if(gGui->layer_above) {
+      if( XA_WIN_LAYER == None )
+	XA_WIN_LAYER = XInternAtom(gGui->display, "_WIN_LAYER", False);
+      
+      data[0] = 10;
+      XChangeProperty(gGui->display, gGui->video_window, XA_WIN_LAYER,
+		      XA_CARDINAL, 32, PropModeReplace, (unsigned char *)data,
+		      1);
+    }
     
-    data[0] = 10;
-    XChangeProperty(gGui->display, gGui->video_window, XA_WIN_LAYER,
-		    XA_CARDINAL, 32, PropModeReplace, (unsigned char *)data,
-		    1);
     XMapRaised (gGui->display, gGui->video_window);
     XUnlockDisplay (gGui->display);
   }
@@ -544,7 +750,6 @@ int video_window_is_visible (void) {
 static unsigned char bm_no_data[] = { 0,0,0,0, 0,0,0,0 };
 void video_window_init (void) {
 
-  XWindowAttributes  attribs;
   Pixmap             bm_no;
   int                x,y,w,h;
 #ifdef HAVE_XINERAMA
@@ -552,9 +757,11 @@ void video_window_init (void) {
   int                   dummy_a, dummy_b;
   XineramaScreenInfo   *screeninfo = NULL;
 #endif
+#ifdef HAVE_XF86VIDMODE
+  int                 dummy_query_event, dummy_query_error;
+#endif
 
-
-  gVw = (gVw_t *) xmalloc(sizeof(gVw_t));
+  gVw = (gVw_t *) xine_xmalloc(sizeof(gVw_t));
 
   gVw->fullscreen_req                   = 0;
   gVw->fullscreen_mode                  = 0;
@@ -565,45 +772,18 @@ void video_window_init (void) {
 
   XLockDisplay (gGui->display);
 
-  XGetWindowAttributes(gGui->display, 
-		       DefaultRootWindow(gGui->display), &attribs);
 
-  gVw->depth = attribs.depth;
-  
-  if (gVw->depth != 15 
-      && gVw->depth != 16 
-      && gVw->depth != 24 
-      && gVw->depth != 32)  {
-    /* The root window may be 8bit but there might still be
-     * visuals with other bit depths. For example this is the 
-     * case on Sun/Solaris machines.
-     */
-    gVw->depth = 24;
-  }
-
-  if (!XMatchVisualInfo(gGui->display, 
-			gGui->screen, gVw->depth, TrueColor, &gVw->vinfo)) {
-    printf ("gui_main: couldn't find true color visual for video window.\n");
-
-    gVw->depth = DefaultDepth (gGui->display, gGui->screen);
-    gVw->vinfo.visual = DefaultVisual (gGui->display, gGui->screen); 
-  }
-  
-  /*
-   * XXX:multivis
-   * To avoid BadMatch errors on XCreateWindow:
-   * If the parent and the new window use different visuals, we must supply a
-   * Colormap
-   */
-  if (gVw->vinfo.visual != DefaultVisual(gGui->display, gGui->screen)) {
-    gVw->colormap = XCreateColormap(gGui->display,
-				    RootWindow(gGui->display, gGui->screen), 
-				    gVw->vinfo.visual, AllocNone);
-  } else {
-    gVw->colormap = DefaultColormap (gGui->display, gGui->screen);
-  }
+  gVw->depth				= gGui->depth;
+  gVw->visual				= gGui->visual;
+  gVw->colormap				= Imlib_get_colormap(gGui->imlib_data);
+  gVw->xwin                             = -8192;
+  gVw->ywin                             = -8192;
+  gVw->desktopWidth                     = DisplayWidth(gGui->display, gGui->screen);
+  gVw->desktopHeight                    = DisplayHeight(gGui->display, gGui->screen);
   
 #ifdef HAVE_XINERAMA
+  gVw->xinerama				= NULL;
+  gVw->xinerama_cnt			= 0;
   /* Spark
    * some Xinerama stuff
    * I want to figure out what fullscreen means for this setup
@@ -621,6 +801,8 @@ void video_window_init (void) {
     if (XineramaIsActive(gGui->display)) {
       gVw->fullscreen_width  = screeninfo[0].width;
       gVw->fullscreen_height = screeninfo[0].height;
+      gVw->xinerama = screeninfo;
+      gVw->xinerama_cnt = screens;
     } else {
       gVw->fullscreen_width  = DisplayWidth  (gGui->display, gGui->screen);
       gVw->fullscreen_height = DisplayHeight (gGui->display, gGui->screen);
@@ -647,7 +829,7 @@ void video_window_init (void) {
    */
 
   bm_no = XCreateBitmapFromData(gGui->display, 
-				DefaultRootWindow(gGui->display), 
+				gGui->imlib_data->x.root, 
 				bm_no_data, 8, 8);
   gVw->cursor[0] = XCreatePixmapCursor(gGui->display, bm_no, bm_no,
 				      &gGui->black, &gGui->black, 0, 0);
@@ -678,13 +860,115 @@ void video_window_init (void) {
     gVw->completion_event = -1;
   }
 
+#ifdef HAVE_XF86VIDMODE
+  if (gGui->config->register_bool (gGui->config, "gui.use_xvidect", 0,
+				   "use XVidModeExtension when switching to fullscreen",
+				   NULL, NULL, NULL)) {
+    if(XF86VidModeQueryExtension(gGui->display, &dummy_query_event, &dummy_query_error)) {
+      XF86VidModeModeInfo* XF86_modelines_swap;
+      int                  major, minor, sort_x, sort_y;
+      
+      XF86VidModeQueryVersion(gGui->display, &major, &minor);
+      printf("XF86VidMode Extension (%d.%d) detected, trying to use it.\n", major, minor);
+      
+      if(XF86VidModeGetAllModeLines(gGui->display, XDefaultScreen(gGui->display), &(gVw->XF86_modelines_count), &(gVw->XF86_modelines))) {
+	printf("XF86VidMode Extension: %d modelines found.\n", gVw->XF86_modelines_count);
+	
+	/*
+	 * sorting modelines, skipping first entry because it is the current
+	 * modeline in use - this is important so we know to which modeline
+	 * we have to switch to when toggling fullscreen mode.
+	 */
+	for(sort_x = 1; sort_x < gVw->XF86_modelines_count; sort_x++) {
+	  for(sort_y = sort_x+1; sort_y < gVw->XF86_modelines_count; sort_y++) {
+	    if(gVw->XF86_modelines[sort_x]->hdisplay > gVw->XF86_modelines[sort_y]->hdisplay) {
+	      XF86_modelines_swap = gVw->XF86_modelines[sort_y];
+	      gVw->XF86_modelines[sort_y] = gVw->XF86_modelines[sort_x];
+	      gVw->XF86_modelines[sort_x] = XF86_modelines_swap;
+	    }
+	  }
+	}
+      } else {
+	gVw->XF86_modelines_count = 0;
+	printf("XF86VidMode Extension: could not get list of available modelines. Failed.\n");
+      }
+    } else {
+      printf("XF86VidMode Extension: initialization failed, not using it.\n");
+    }
+  }
+#endif
+
   XUnlockDisplay (gGui->display);
 
-  video_window_adapt_size (768, 480, &x, &y, &w, &h);
+  video_window_adapt_size (NULL, 768, 480, &x, &y, &w, &h);
   video_window_draw_logo();
 
 }
 
+
+
+/*
+ * Translate screen coordinates to video coordinates
+ */
+static int video_window_translate_point(int gui_x, int gui_y,
+					int *video_x, int *video_y)
+{
+  x11_rectangle_t rect;
+  int xwin, ywin;
+  unsigned int wwin, hwin, bwin, dwin;
+  float xf,yf;
+  float scale, width_scale, height_scale,aspect;
+  Window rootwin;
+
+  rect.x = gui_x;
+  rect.y = gui_y;
+  rect.w = 0;
+  rect.h = 0;
+
+  if (gGui->vo_driver->gui_data_exchange (gGui->vo_driver,
+					  GUI_DATA_EX_TRANSLATE_GUI_TO_VIDEO, 
+					  (void*)&rect) != -1) {
+    /* driver implements gui->video coordinate space translation, use it */
+    *video_x = rect.x;
+    *video_y = rect.y;
+    return 1;
+  }
+
+  /* Driver cannot convert gui->video space, fall back to old code... */
+
+  if(XGetGeometry(gGui->display, gGui->video_window, &rootwin, 
+		  &xwin, &ywin, &wwin, &hwin, &bwin, &dwin) == BadDrawable)
+    return 0;
+
+  /* Scale co-ordinate to image dimensions. */
+  height_scale=(float)gVw->video_height/(float)hwin;
+  width_scale=(float)gVw->video_width/(float)wwin;
+  aspect=(float)gVw->video_width/(float)gVw->video_height;
+  if (((float)wwin/(float)hwin)<aspect) {
+    scale=width_scale;
+    xf=(float)gui_x * scale;
+    yf=(float)gui_y * scale;
+    //wwin=wwin * scale;
+    hwin=hwin * scale;
+    /* FIXME: The 1.25 should really come from the NAV packets. */
+    *video_x=xf * 1.25 / aspect;
+    *video_y=yf-((hwin-gVw->video_height)/2);
+    /* printf("wscale:a=%f, s=%f, x=%d, y=%d\n",aspect, scale,*video_x,*video_y);  */
+  } else {
+    scale=height_scale;
+    xf=(float)gui_x * scale;
+    yf=(float)gui_y * scale;
+    wwin=wwin * scale;
+    /* FIXME: The 1.25 should really come from the NAV packets. */
+    *video_x=(xf-((wwin-gVw->video_width)/2)) * 1.25 / aspect;
+    *video_y=yf;
+    /* printf("hscale:a=%f s=%f x=%d, y=%d\n",aspect,scale,*video_x,*video_y);  */
+  }
+
+  return 1;
+}
+
+
 /*
  *
  */
@@ -692,6 +976,11 @@ static void video_window_handle_event (XEvent *event, void *data) {
 
   switch(event->type) {
 
+  case DestroyNotify:
+    if(gGui->video_window == event->xany.window)
+      gui_exit(NULL, NULL);
+    break;
+
   case KeyPress:
 
     if(!gGui->cursor_visible) {
@@ -704,38 +993,30 @@ static void video_window_handle_event (XEvent *event, void *data) {
 
   case MotionNotify: {
     XMotionEvent *mevent = (XMotionEvent *) event;
-    mouse_event_t xine_event;
-    int xwin, ywin;
-    unsigned int wwin, hwin, bwin, dwin;
-    float xf,yf;
-    Window rootwin;
+    xine_input_event_t xine_event;
+    int x, y;
+
+    /* printf("Mouse event:mx=%d my=%d\n",mevent->x, mevent->y); */
     
     if(!gGui->cursor_visible) {
       gGui->cursor_visible = !gGui->cursor_visible;
       video_window_set_cursor_visibility(gGui->cursor_visible);
     }
 
-    if(XGetGeometry(gGui->display, gGui->video_window, &rootwin, 
-		    &xwin, &ywin, &wwin, &hwin, &bwin, &dwin) != BadDrawable) {
-      xine_event.event.type = XINE_MOUSE_EVENT;
-      xine_event.button = 0; // No buttons, just motion.
-      /* Scale co-ordinate to image dimensions. */
-      xf = (float)mevent->x / (float)wwin;
-      yf = (float)mevent->y / (float)hwin;
-      xine_event.x = (uint16_t)( xf * gVw->video_width ); 
-      xine_event.y = (uint16_t)( yf * gVw->video_height );
-      xine_send_event(gGui->xine, (event_t*)(&xine_event), NULL);
+    if (video_window_translate_point(mevent->x, mevent->y, &x, &y)) {
+      xine_event.event.type = XINE_EVENT_MOUSE_MOVE;
+      xine_event.button = 0; /*  No buttons, just motion. */
+      xine_event.x = x;
+      xine_event.y = y;
+      xine_send_event(gGui->xine, (xine_event_t*)(&xine_event));
     }
   }
   break;
 
   case ButtonPress: {
     XButtonEvent *bevent = (XButtonEvent *) event;
-    mouse_event_t xine_event;
-    int xwin, ywin;
-    unsigned int wwin, hwin, bwin, dwin;
-    float xf,yf;
-    Window rootwin;
+    xine_input_event_t xine_event;
+    int x, y;
 
     if(!gGui->cursor_visible) {
       gGui->cursor_visible = !gGui->cursor_visible;
@@ -746,16 +1027,12 @@ static void video_window_handle_event (XEvent *event, void *data) {
       panel_toggle_visibility(NULL, NULL);
 
     if (bevent->button == Button1) {
-      if(XGetGeometry(gGui->display, gGui->video_window, &rootwin, 
-		      &xwin, &ywin, &wwin, &hwin, &bwin, &dwin) != BadDrawable) {
-	xine_event.event.type = XINE_MOUSE_EVENT;
+      if (video_window_translate_point(bevent->x, bevent->y, &x, &y)) {
+	xine_event.event.type = XINE_EVENT_MOUSE_BUTTON;
 	xine_event.button = 1;
-	/* Scale co-ordinate to image dimensions. */
-	xf = (float)bevent->x / (float)wwin;
-	yf = (float)bevent->y / (float)hwin;
-	xine_event.x = (uint16_t)( xf * gVw->video_width ); 
-	xine_event.y = (uint16_t)( yf * gVw->video_height );
-	xine_send_event(gGui->xine, (event_t*)(&xine_event), NULL);
+	xine_event.x = x;
+	xine_event.y = y;
+	xine_send_event(gGui->xine, (xine_event_t*)(&xine_event));
       }
     }
   }
