@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: inputtext.c,v 1.8 2001/08/26 20:17:08 f1rmb Exp $
+ * $Id: inputtext.c,v 1.20 2002/01/15 22:22:36 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -36,6 +36,7 @@
 #include "widget.h"
 #include "image.h"
 #include "inputtext.h"
+#include "font.h"
 #include "widget_types.h"
 
 #include "_xitk.h"
@@ -43,6 +44,60 @@
 #define NORMAL                  1
 #define FOCUS                   2
 
+/*
+ *
+ */
+static void notify_destroy(xitk_widget_t *w, void *data) {
+  inputtext_private_data_t *private_data = 
+    (inputtext_private_data_t *) w->private_data;
+
+  if(w->widget_type & WIDGET_TYPE_INPUTTEXT) {
+    XITK_FREE(private_data->skin_element_name);
+    xitk_image_free_image(private_data->imlibdata, &private_data->skin);
+    XITK_FREE(private_data->text);
+    XITK_FREE(private_data->normal_color);
+    XITK_FREE(private_data->focused_color);
+    XITK_FREE(private_data->fontname);
+    XITK_FREE(private_data);
+  }
+}
+
+/*
+ *
+ */
+static xitk_image_t *get_skin(xitk_widget_t *w, int sk) {
+  inputtext_private_data_t *private_data = 
+    (inputtext_private_data_t *) w->private_data;
+  
+  if(w->widget_type & WIDGET_TYPE_INPUTTEXT) {
+    if(sk == BACKGROUND_SKIN && private_data->skin) {
+      return private_data->skin;
+    }
+  }
+
+  return NULL;
+}
+
+/*
+ *
+ */
+static int notify_inside(xitk_widget_t *it, int x, int y) {
+  inputtext_private_data_t *private_data = 
+    (inputtext_private_data_t *) it->private_data;
+  
+  if(it->widget_type & WIDGET_TYPE_INPUTTEXT) {
+    if(it->visible) {
+      xitk_image_t *skin = private_data->skin;
+      
+      return xitk_is_cursor_out_mask(private_data->imlibdata->x.disp, it, skin->mask, x, y);
+    }
+    else
+      return 0;
+  }
+
+  return 1;
+}
+
 /*
  * Extract modifier keys.
  */
@@ -101,49 +156,88 @@ int xitk_get_key_modifier(XEvent *xev, int *modifier) {
   return (*modifier != MODIFIER_NOMOD);
 }
 
+/*
+ * Recalculate display offsets.
+ */
+static void inputtext_recalc_offsets(xitk_widget_list_t *wl, 
+				     inputtext_private_data_t *private_data) {
+  char         *label = private_data->text;
+  char         *p = private_data->text;
+  xitk_font_t  *fs = NULL;
+  int           width;
+  int           i = 0;
+
+  if(private_data->cursor_pos >= 0) {
+    
+    if(private_data->fontname)
+      fs = xitk_font_load_font(private_data->imlibdata->x.disp, private_data->fontname);
+
+    if(fs == NULL) 
+      fs = xitk_font_load_font(private_data->imlibdata->x.disp, xitk_get_system_font());
+
+    if(fs == NULL)
+      XITK_DIE("%s()@%d: xitk_font_load_font() failed. Exiting\n", __FUNCTION__, __LINE__);
+
+    xitk_font_set_font(fs, wl->gc);
+
+    private_data->disp_offset = private_data->pos_in_pos = 0;
+    
+    while(i < private_data->cursor_pos) {
+      
+      p = label;
+      p += private_data->disp_offset;
+      
+      width = xitk_font_get_string_length(fs, p);
+
+      if(((i == (strlen(label))) && (width > private_data->max_visible)) 
+	 || ((xitk_font_get_text_width(fs, p, 
+				       (i - private_data->disp_offset))) 
+	     >= private_data->max_visible)) {
+	
+	private_data->pos[++private_data->pos_in_pos] = i - 1;
+	private_data->disp_offset = private_data->pos[private_data->pos_in_pos];
+      }
+
+      i++;
+    }
+
+    xitk_font_unload_font(fs);
+  }
+}
+
 /*
  * Return a pixmap with text drawed.
  */
-static Pixmap create_labelofinputtext(widget_t *it, 
+static Pixmap create_labelofinputtext(xitk_widget_t *it, 
 				      Window win, GC gc, Pixmap pix, 
 				      int xsize, int ysize, 
 				      char *label, int state) {
   inputtext_private_data_t *private_data = 
     (inputtext_private_data_t *) it->private_data;
   char               *plabel = label;
-  XFontStruct        *fs = NULL;
-  XCharStruct         cs;
-  int                 dir, as, des, len, yoff = 0, DefaultColor = -1;
+  char               *p = label;
+  xitk_font_t        *fs = NULL;
+  int                 lbear, rbear, width, asc, des;
+  int                 yoff = 0, DefaultColor = -1;
   unsigned int        fg;
   XColor              color;
-  gui_color_names_t  *gColor = NULL;
+  xitk_color_names_t  *gColor = NULL;
 
   color.flags = DoRed|DoBlue|DoGreen;
 
   /* Try to load font */
-  /*
+
   if(private_data->fontname)
-    fs = XLoadQueryFont(private_data->display, private_data->fontname);
-  */
-  if(fs == NULL) fs = XLoadQueryFont(private_data->display, "fixed");
-  if(fs == NULL) fs = XLoadQueryFont(private_data->display, "times-roman");
-  if(fs == NULL) fs = XLoadQueryFont(private_data->display, "*times-medium-r*");
+    fs = xitk_font_load_font(private_data->imlibdata->x.disp, private_data->fontname);
 
-  if(fs == NULL) {
-    fprintf(stderr, "%s()@%d: XLoadQueryFont() returned NULL!. Exiting\n",
-	    __FUNCTION__, __LINE__);
-    exit(1);
-  }
+  if(fs == NULL) 
+    fs = xitk_font_load_font(private_data->imlibdata->x.disp, xitk_get_system_font());
 
-  XLOCK(private_data->display);
+  if(fs == NULL)
+    XITK_DIE("%s()@%d: xitk_font_load_font() failed. Exiting\n", __FUNCTION__, __LINE__);
 
-  XSetFont(private_data->display, gc, fs->fid);
-
-  XTextExtents(fs, label, strlen(label), &dir, &as, &des, &cs);
-
-  XUNLOCK(private_data->display);
-
-  len = cs.width;
+  xitk_font_set_font(fs, gc);
+  xitk_font_string_extent(fs, label, &lbear, &rbear, &width, &asc, &des);
 
   /*  Some colors configurations */
   switch(state) {
@@ -152,7 +246,7 @@ static Pixmap create_labelofinputtext(widget_t *it,
       DefaultColor = 0;
     }
     else {
-      gColor = gui_get_color_name(private_data->normal_color);
+      gColor = xitk_get_color_name(private_data->normal_color);
     }
     break;
 
@@ -161,7 +255,7 @@ static Pixmap create_labelofinputtext(widget_t *it,
       DefaultColor = 0;
     }
     else {
-      gColor = gui_get_color_name(private_data->focused_color);
+      gColor = xitk_get_color_name(private_data->focused_color);
     }
     break;
     
@@ -176,66 +270,77 @@ static Pixmap create_labelofinputtext(widget_t *it,
     color.green = gColor->green<<8;
   }
 
-  XLOCK(private_data->display);
+  XLOCK(private_data->imlibdata->x.disp);
 
-  XAllocColor(private_data->display,
-	      DefaultColormap(private_data->display, 0), &color);
+  XAllocColor(private_data->imlibdata->x.disp,
+	      Imlib_get_colormap(private_data->imlibdata), &color);
   fg = color.pixel;
   
-  XSetForeground(private_data->display, gc, fg);
+  XSetForeground(private_data->imlibdata->x.disp, gc, fg);
 
-  XUNLOCK(private_data->display);
+  XUNLOCK(private_data->imlibdata->x.disp);
 
-  if((private_data->cursor_pos - private_data->disp_offset)
-     > private_data->max_visible) {
-    private_data->disp_offset += private_data->max_visible - 1;
-  }
-  else if(private_data->disp_offset > private_data->cursor_pos) {
-    private_data->disp_offset -= private_data->max_visible;
-    if(private_data->disp_offset < 0)
-      private_data->disp_offset = 0;
+  if(private_data->cursor_pos >= 0) {
+
+    if(private_data->disp_offset)
+      p += private_data->disp_offset;
+    
+    width = xitk_font_get_string_length(fs, p);
+    
+    if(private_data->disp_offset > private_data->cursor_pos && private_data->pos_in_pos) {
+      
+      private_data->disp_offset = private_data->pos[--private_data->pos_in_pos];
+      
+      if(private_data->disp_offset < 0) {
+	private_data->pos[private_data->pos_in_pos] = private_data->disp_offset = 0;
+      }
+      
+    }
+    else if(((private_data->cursor_pos == (strlen(label)))
+	     && (width > private_data->max_visible))
+	    || ((xitk_font_get_text_width(fs, p, 
+					  (private_data->cursor_pos - private_data->disp_offset)))
+		>= private_data->max_visible)) {
+      
+      private_data->pos[++private_data->pos_in_pos] = private_data->cursor_pos - 1;
+      private_data->disp_offset = private_data->pos[private_data->pos_in_pos];
+    }
   }
 
   if(private_data->disp_offset)
     plabel += private_data->disp_offset;
 
   /*  Put text in the right place */
-  XLOCK(private_data->display);
+  XLOCK(private_data->imlibdata->x.disp);
 
-  XDrawString(private_data->display, pix, gc, 
-	      2, ((ysize+as+des+yoff)>>1)-des, 
+  XDrawString(private_data->imlibdata->x.disp, pix, gc, 
+	      2, ((ysize+asc+des+yoff)>>1)-des, 
 	      plabel, strlen(plabel));
 
+  width = xitk_font_get_text_width(fs, plabel,
+				   (private_data->cursor_pos - private_data->disp_offset));
+  
   /* Draw cursor pointer */
   if(private_data->cursor_pos >= 0) {
     
-    XDrawLine(private_data->display, pix, gc,
-	      (6 * (private_data->cursor_pos - private_data->disp_offset)) + 1,
-	      2, 
-	      (6 * (private_data->cursor_pos - private_data->disp_offset)) + 3,
-	      2);
+    XDrawLine(private_data->imlibdata->x.disp, pix, gc,
+	      width + 1, 2, width + 3, 2);
 
-    XDrawLine(private_data->display, pix, gc, 
-	      (6 * (private_data->cursor_pos - private_data->disp_offset)) + 2
-	      , 2, 
-	      (6 * (private_data->cursor_pos - private_data->disp_offset)) + 2,
-	      ysize - 3);
+    XDrawLine(private_data->imlibdata->x.disp, pix, gc, 
+	      width + 2, 2, width + 2, ysize - 3);
 
-    XDrawLine(private_data->display, pix, gc, 
-	      (6 * (private_data->cursor_pos - private_data->disp_offset)) + 1,
-	      ysize - 3, 
-	      (6 * (private_data->cursor_pos - private_data->disp_offset)) + 3,
-	      ysize - 3);
+    XDrawLine(private_data->imlibdata->x.disp, pix, gc, 
+	      width + 1, ysize - 3, width + 3, ysize - 3);
 
   }
   
-  XFreeFont(private_data->display, fs);
+  XUNLOCK(private_data->imlibdata->x.disp);
 
-  XUNLOCK(private_data->display);
+  xitk_font_unload_font(fs);
 
   if(gColor) {
-    free(gColor->colorname);
-    free(gColor);
+    XITK_FREE(gColor->colorname);
+    XITK_FREE(gColor);
   }
 
   return pix;
@@ -244,36 +349,45 @@ static Pixmap create_labelofinputtext(widget_t *it,
 /*
  * Paint the input text box.
  */
-static void paint_inputtext(widget_t *it, Window win, GC gc) {
+static void paint_inputtext(xitk_widget_t *it, Window win, GC gc) {
   inputtext_private_data_t *private_data = 
     (inputtext_private_data_t *) it->private_data;
-  int          button_width, state = 0;
-  gui_image_t *skin;
-  Pixmap       btn, bgtmp;
-  XWindowAttributes attr;
+  int                 button_width, state = 0;
+  xitk_image_t        *skin;
+  Pixmap              btn, bgtmp;
+  GC                  lgc;
+  XWindowAttributes   attr;
 
   if ((it->widget_type & WIDGET_TYPE_INPUTTEXT) && it->visible) {
         
-    XGetWindowAttributes(private_data->display, win, &attr);
+    XGetWindowAttributes(private_data->imlibdata->x.disp, win, &attr);
     
     skin = private_data->skin;
     
+    lgc = XCreateGC(private_data->imlibdata->x.disp, win, None, None);
+    XCopyGC(private_data->imlibdata->x.disp, gc, (1 << GCLastBit) - 1, lgc);
+
+    if (skin->mask) {
+      XSetClipOrigin(private_data->imlibdata->x.disp, lgc, it->x, it->y);
+      XSetClipMask(private_data->imlibdata->x.disp, lgc, skin->mask);
+    }
+
     button_width = skin->width/2;
     
-    bgtmp = XCreatePixmap(private_data->display, skin->image,
+    bgtmp = XCreatePixmap(private_data->imlibdata->x.disp, skin->image,
 			  button_width, skin->height, attr.depth);
     
-    XLOCK(private_data->display);
+    XLOCK(private_data->imlibdata->x.disp);
     
     if(private_data->have_focus) {
       state = FOCUS;
-      XCopyArea (private_data->display, skin->image,
+      XCopyArea (private_data->imlibdata->x.disp, skin->image,
 		 bgtmp, gc, button_width, 0,
 		 button_width, skin->height, 0, 0);
     }
     else {
       state = NORMAL;
-      XCopyArea (private_data->display, skin->image,
+      XCopyArea (private_data->imlibdata->x.disp, skin->image,
 		 bgtmp, gc, 0, 0,
 		 button_width, skin->height, 0, 0);
     }
@@ -283,25 +397,23 @@ static void paint_inputtext(widget_t *it, Window win, GC gc) {
 				  button_width, skin->height, 
 				  private_data->text, state);
     
-    XCopyArea (private_data->display, btn, win, gc, 0, 0,
+    XCopyArea (private_data->imlibdata->x.disp, btn, win, lgc, 0, 0,
 	       button_width, skin->height, it->x, it->y);
 
 
-    XFreePixmap(private_data->display, bgtmp);
+    XFreePixmap(private_data->imlibdata->x.disp, bgtmp);
 
-    XUNLOCK(private_data->display);
+    XFreeGC(private_data->imlibdata->x.disp, lgc);
+
+    XUNLOCK(private_data->imlibdata->x.disp);
   }
-#ifdef DEBUG_GUI
-  else
-    fprintf (stderr, "paint input text on something (%d) "
-	     "that is not an input text\n", it->widget_type);
-#endif
+
 }
 
 /*
  * Handle click events.
  */
-static int notify_click_inputtext(widget_list_t *wl, widget_t *it, 
+static int notify_click_inputtext(xitk_widget_list_t *wl, xitk_widget_t *it, 
 				   int bUp, int x, int y) {
   inputtext_private_data_t *private_data = 
     (inputtext_private_data_t *) it->private_data;
@@ -314,8 +426,34 @@ static int notify_click_inputtext(widget_list_t *wl, widget_t *it,
     }
 
     pos = x - it->x;
-    pos /= 6;
+    
+    {
+      char        *p = private_data->text;
+      xitk_font_t *fs = NULL;
+      int          width = 0, i = 0;
+      
+      p += private_data->disp_offset;
 
+      if(private_data->fontname)
+	fs = xitk_font_load_font(private_data->imlibdata->x.disp, private_data->fontname);
+      
+      if(fs == NULL) 
+	fs = xitk_font_load_font(private_data->imlibdata->x.disp, xitk_get_system_font());
+      
+      if(fs == NULL)
+	XITK_DIE("%s()@%d: xitk_font_load_font() failed. Exiting\n", __FUNCTION__, __LINE__);
+      
+      xitk_font_set_font(fs, wl->gc);
+      
+      while(width < pos) {
+	
+	width = xitk_font_get_text_width(fs, p, i);
+	i++;
+      }
+      xitk_font_unload_font(fs);
+      pos = (i-2) + private_data->disp_offset;
+    }
+    
     if(private_data->text) {
       if(pos > strlen(private_data->text))
 	pos = strlen(private_data->text);
@@ -323,16 +461,11 @@ static int notify_click_inputtext(widget_list_t *wl, widget_t *it,
     else
       pos = 0;
 
-    private_data->cursor_pos = pos;
+    private_data->cursor_pos = (pos < 0) ? 0 : pos;
 
     paint_inputtext(it, wl->win, wl->gc);
 
   }
-#ifdef DEBUG_GUI
-  else
-    fprintf (stderr, "notify focus input text on something (%d) "
-	     "that is not an input text\n", it->widget_type);
-#endif
 
   return 1;
 }
@@ -340,35 +473,61 @@ static int notify_click_inputtext(widget_list_t *wl, widget_t *it,
 /*
  * Handle motion on input text box.
  */
-static int notify_focus_inputtext(widget_list_t *wl, 
-				   widget_t *it, int bEntered) {
+static int notify_focus_inputtext(xitk_widget_list_t *wl, 
+				   xitk_widget_t *it, int bEntered) {
   inputtext_private_data_t *private_data = 
     (inputtext_private_data_t *) it->private_data;
   
   if (it->widget_type & WIDGET_TYPE_INPUTTEXT) {
     private_data->have_focus = bEntered;
     if(bEntered == FOCUS_LOST) {
-      private_data->cursor_pos = -1;
-    }
-    else {
-      private_data->cursor_pos = strlen(private_data->text);
+      private_data->cursor_pos = 0;
     }
     paint_inputtext(it, wl->win, wl->gc);
 
   }
-#ifdef DEBUG_GUI
-  else
-    fprintf(stderr, "notify focus input text on something (%d) "
-	    "that is not an input text\n", it->widget_type);
-#endif
 
   return 1;
 }
 
+/*
+ *
+ */
+static void notify_change_skin(xitk_widget_list_t *wl, 
+			       xitk_widget_t *it, xitk_skin_config_t *skonfig) {
+  inputtext_private_data_t *private_data = 
+    (inputtext_private_data_t *) it->private_data;
+  
+  if (it->widget_type & WIDGET_TYPE_INPUTTEXT) {
+    if(private_data->skin_element_name) {
+      XITK_FREE(private_data->fontname);
+      private_data->fontname      = xitk_skin_get_label_fontname(skonfig, private_data->skin_element_name);
+      
+      XITK_FREE_XITK_IMAGE(private_data->imlibdata->x.disp, private_data->skin);
+      private_data->skin          = xitk_image_load_image(private_data->imlibdata, 
+							  xitk_skin_get_skin_filename(skonfig, private_data->skin_element_name));
+      
+      private_data->max_visible   = (private_data->skin->width/2);
+      
+      XITK_FREE(private_data->normal_color);
+      private_data->normal_color  = xitk_skin_get_label_color(skonfig, private_data->skin_element_name);
+      XITK_FREE(private_data->focused_color);
+      private_data->focused_color = xitk_skin_get_label_color_focus(skonfig, private_data->skin_element_name);
+      
+      it->x                       = xitk_skin_get_coord_x(skonfig, private_data->skin_element_name);
+      it->y                       = xitk_skin_get_coord_y(skonfig, private_data->skin_element_name);
+      it->width                   = private_data->skin->width/2;
+      it->height                  = private_data->skin->height;
+      
+      xitk_set_widget_pos(it, it->x, it->y);
+    }
+  }
+}
+
 /*
  * Erase one char from right of cursor.
  */
-static void inputtext_erase_with_delete(widget_list_t *wl, widget_t *it) {
+static void inputtext_erase_with_delete(xitk_widget_list_t *wl, xitk_widget_t *it) {
   inputtext_private_data_t *private_data = 
     (inputtext_private_data_t *) it->private_data;
   char *oldtext, *newtext;
@@ -380,7 +539,7 @@ static void inputtext_erase_with_delete(widget_list_t *wl, widget_t *it) {
        && (private_data->cursor_pos < strlen(private_data->text))) {
       
       oldtext = strdup(private_data->text);
-      newtext = (char *) gui_xmalloc(strlen(oldtext));
+      newtext = (char *) xitk_xmalloc(strlen(oldtext));
       
       offset = 0;
       p = oldtext;
@@ -403,13 +562,13 @@ static void inputtext_erase_with_delete(widget_list_t *wl, widget_t *it) {
       
       *pp = 0;
       
-      free(private_data->text);
+      XITK_FREE(private_data->text);
       private_data->text = strdup((newtext != NULL) ? newtext : "");
       
       paint_inputtext(it, wl->win, wl->gc);
       
-      free(oldtext);
-      free(newtext);
+      XITK_FREE(oldtext);
+      XITK_FREE(newtext);
     }
   }
   else {
@@ -421,7 +580,7 @@ static void inputtext_erase_with_delete(widget_list_t *wl, widget_t *it) {
 /*
  * Erase one char from left of cursor.
  */
-static void inputtext_erase_with_backspace(widget_list_t *wl, widget_t *it) {
+static void inputtext_erase_with_backspace(xitk_widget_list_t *wl, xitk_widget_t *it) {
   inputtext_private_data_t *private_data = 
     (inputtext_private_data_t *) it->private_data;
   char *oldtext, *newtext;
@@ -432,7 +591,7 @@ static void inputtext_erase_with_backspace(widget_list_t *wl, widget_t *it) {
     if(private_data->cursor_pos > 0) {
       
       oldtext = strdup(private_data->text);
-      newtext = (char *) gui_xmalloc(strlen(oldtext));
+      newtext = (char *) xitk_xmalloc(strlen(oldtext));
       
       private_data->cursor_pos--;
       
@@ -456,13 +615,13 @@ static void inputtext_erase_with_backspace(widget_list_t *wl, widget_t *it) {
       } 
       *pp = 0;
       
-      free(private_data->text);
+      XITK_FREE(private_data->text);
       private_data->text = strdup(newtext);
       
       paint_inputtext(it, wl->win, wl->gc);
       
-      free(oldtext);
-      free(newtext);
+      XITK_FREE(oldtext);
+      XITK_FREE(newtext);
     }
   }
   else {
@@ -475,7 +634,7 @@ static void inputtext_erase_with_backspace(widget_list_t *wl, widget_t *it) {
 /*
  * Erase chars from cursor pos to EOL.
  */
-static void inputtext_kill_line(widget_list_t *wl, widget_t *it) {
+static void inputtext_kill_line(xitk_widget_list_t *wl, xitk_widget_t *it) {
   inputtext_private_data_t *private_data = 
     (inputtext_private_data_t *) it->private_data;
   char *oldtext, *newtext;
@@ -484,17 +643,17 @@ static void inputtext_kill_line(widget_list_t *wl, widget_t *it) {
     if(private_data->cursor_pos >= 0) {
 
       oldtext = strdup(private_data->text);
-      newtext = (char *) gui_xmalloc(strlen(oldtext));
+      newtext = (char *) xitk_xmalloc(strlen(oldtext));
 
       snprintf(newtext, private_data->cursor_pos + 1, "%s", oldtext);
 
-      free(private_data->text);
+      XITK_FREE(private_data->text);
       private_data->text = strdup(newtext);
       
       paint_inputtext(it, wl->win, wl->gc);
       
-      free(oldtext);
-      free(newtext);
+      XITK_FREE(oldtext);
+      XITK_FREE(newtext);
     }
   }
 }
@@ -502,7 +661,7 @@ static void inputtext_kill_line(widget_list_t *wl, widget_t *it) {
 /*
  * Move cursor pos to left.
  */
-static void inputtext_move_left(widget_list_t *wl, widget_t *it) {
+static void inputtext_move_left(xitk_widget_list_t *wl, xitk_widget_t *it) {
   inputtext_private_data_t *private_data = 
     (inputtext_private_data_t *) it->private_data;
   
@@ -515,7 +674,7 @@ static void inputtext_move_left(widget_list_t *wl, widget_t *it) {
 /*
  * Move cursor pos to right.
  */
-static void inputtext_move_right(widget_list_t *wl, widget_t *it) {
+static void inputtext_move_right(xitk_widget_list_t *wl, xitk_widget_t *it) {
   inputtext_private_data_t *private_data = 
     (inputtext_private_data_t *) it->private_data;
 
@@ -528,7 +687,7 @@ static void inputtext_move_right(widget_list_t *wl, widget_t *it) {
 /*
  * Remove focus of widget, then call callback function.
  */
-static void inputtext_exec_return(widget_list_t *wl, widget_t *it) {
+static void inputtext_exec_return(xitk_widget_list_t *wl, xitk_widget_t *it) {
   inputtext_private_data_t *private_data = 
     (inputtext_private_data_t *) it->private_data;
 
@@ -546,7 +705,7 @@ static void inputtext_exec_return(widget_list_t *wl, widget_t *it) {
 /*
  * Remove focus of widget.
  */
-static void inputtext_exec_escape(widget_list_t *wl, widget_t *it) {
+static void inputtext_exec_escape(xitk_widget_list_t *wl, xitk_widget_t *it) {
   inputtext_private_data_t *private_data = 
     (inputtext_private_data_t *) it->private_data;
   
@@ -558,12 +717,13 @@ static void inputtext_exec_escape(widget_list_t *wl, widget_t *it) {
 /*
  *
  */
-static void inputtext_move_bol(widget_list_t *wl, widget_t *it) {
+static void inputtext_move_bol(xitk_widget_list_t *wl, xitk_widget_t *it) {
   inputtext_private_data_t *private_data = 
     (inputtext_private_data_t *) it->private_data;
 
   if(private_data->text) {
-    private_data->cursor_pos = 0;
+    private_data->cursor_pos = private_data->disp_offset = 0;
+    inputtext_recalc_offsets(wl, private_data);
     paint_inputtext(it, wl->win, wl->gc);
   }
 }
@@ -571,21 +731,42 @@ static void inputtext_move_bol(widget_list_t *wl, widget_t *it) {
 /*
  *
  */
-static void inputtext_move_eol(widget_list_t *wl, widget_t *it) {
+static void inputtext_move_eol(xitk_widget_list_t *wl, xitk_widget_t *it) {
   inputtext_private_data_t *private_data = 
     (inputtext_private_data_t *) it->private_data;
 
   if(private_data->text) {
     private_data->cursor_pos = strlen(private_data->text);
+    inputtext_recalc_offsets(wl, private_data);
+    
     paint_inputtext(it, wl->win, wl->gc);
   }
 }
 
+/*
+ * Transpose two characters.
+ */
+static void inputtext_transpose_chars(xitk_widget_list_t *wl, xitk_widget_t *it) {
+  inputtext_private_data_t *private_data = 
+    (inputtext_private_data_t *) it->private_data;
+  
+  if(private_data->text && (strlen(private_data->text) >= 2)) {
+    if((private_data->cursor_pos >= 2)) {
+      int c = private_data->text[private_data->cursor_pos - 2];
+
+      private_data->text[private_data->cursor_pos - 2] = private_data->text[private_data->cursor_pos - 1];
+      private_data->text[private_data->cursor_pos - 1] = c;
+      
+      paint_inputtext(it, wl->win, wl->gc);
+    }
+  }
+}
+
 /*
  * Handle keyboard event in input text box.
  */
-static void notify_keyevent_inputtext(widget_list_t *wl,
-				      widget_t *it, XEvent *xev) {
+static void notify_keyevent_inputtext(xitk_widget_list_t *wl,
+				      xitk_widget_t *it, XEvent *xev) {
   inputtext_private_data_t *private_data = 
     (inputtext_private_data_t *) it->private_data;
   
@@ -596,9 +777,9 @@ static void notify_keyevent_inputtext(widget_list_t *wl,
     int         len;
     int         modifier;
     
-    XLOCK(private_data->display);
+    XLOCK(private_data->imlibdata->x.disp);
     len = XLookupString(&keyevent, buf, sizeof(buf), &key, NULL);
-    XUNLOCK(private_data->display);
+    XUNLOCK(private_data->imlibdata->x.disp);
 
     buf[len] = '\0';
 
@@ -678,6 +859,14 @@ static void notify_keyevent_inputtext(widget_list_t *wl,
 	}
 	break;
 
+	/* Transpose chars */
+      case XK_t:
+      case XK_T:
+	if(modifier & MODIFIER_CTRL) {
+	  inputtext_transpose_chars(wl, it);
+	}
+	break;
+
       case XK_question:
 	if(modifier & MODIFIER_CTRL) {
 	  inputtext_erase_with_backspace(wl, it);
@@ -724,8 +913,8 @@ static void notify_keyevent_inputtext(widget_list_t *wl,
 	else
 	  pos = private_data->cursor_pos;
 
-	free(private_data->text);
-	private_data->text = (char *) gui_xmalloc(strlen(oldtext) + 3);
+	XITK_FREE(private_data->text);
+	private_data->text = (char *) xitk_xmalloc(strlen(oldtext) + 3);
 
 	sprintf(private_data->text, "%s", oldtext);
 	sprintf(&private_data->text[pos], "%c%s%c",
@@ -733,7 +922,7 @@ static void notify_keyevent_inputtext(widget_list_t *wl,
 
 	private_data->cursor_pos++;
 	paint_inputtext(it, wl->win, wl->gc);
-	free(oldtext);
+	XITK_FREE(oldtext);
       }
       
     }
@@ -741,29 +930,19 @@ static void notify_keyevent_inputtext(widget_list_t *wl,
       //      printf("got unhandled key = [%ld]\n", key);
     }
   }
-#ifdef DEBUG_GUI
-  else
-    fprintf (stderr, "notify key event input text on something (%d) "
-	     "that is not an input text\n", it->widget_type);
-#endif
 
 }
 
 /*
  * Return the text of widget.
  */
-char *inputttext_get_text(widget_t *it) {
+char *xitk_inputtext_get_text(xitk_widget_t *it) {
   inputtext_private_data_t *private_data = 
     (inputtext_private_data_t *) it->private_data;
   
   if (it->widget_type & WIDGET_TYPE_INPUTTEXT) {
     return private_data->text;
   }
-#ifdef DEBUG_GUI
-  else
-    fprintf(stderr, "change input text on something (%d) "
-	    "that is not an input text\n", it->widget_type);
-#endif
   
   return NULL;
 }
@@ -771,73 +950,118 @@ char *inputttext_get_text(widget_t *it) {
 /*
  * Change and redisplay the text of widget.
  */
-void inputtext_change_text(widget_list_t *wl, widget_t *it, char *text) {
+void xitk_inputtext_change_text(xitk_widget_list_t *wl, xitk_widget_t *it, char *text) {
   inputtext_private_data_t *private_data = 
     (inputtext_private_data_t *) it->private_data;
   
   if (it->widget_type & WIDGET_TYPE_INPUTTEXT) {
-    free(private_data->text);
+    XITK_FREE(private_data->text);
     private_data->text = strdup((text != NULL)?text:"");
     private_data->cursor_pos = -1;
     paint_inputtext(it, wl->win, wl->gc);
   }
-#ifdef DEBUG_GUI
-  else
-    fprintf(stderr, "change input text on something (%d) "
-	    "that is not an input text\n", it->widget_type);
-#endif
+
 }
 
 /*
  * Create input text box
  */
-widget_t *inputtext_create (xitk_inputtext_t *it) {
-  widget_t                  *mywidget;
+static xitk_widget_t *_xitk_inputtext_create (xitk_skin_config_t *skonfig, 
+					      xitk_inputtext_widget_t *it,
+					      int x, int y, char *skin_element_name,
+					      xitk_image_t *skin,
+					      char *fontname,
+					      char *ncolor, char *fcolor) {
+  xitk_widget_t             *mywidget;
   inputtext_private_data_t  *private_data;
   
-  mywidget = (widget_t *) gui_xmalloc (sizeof(widget_t));
+  mywidget                        = (xitk_widget_t *) xitk_xmalloc (sizeof(xitk_widget_t));
   
-  private_data                = (inputtext_private_data_t *) 
-    gui_xmalloc (sizeof(inputtext_private_data_t));
+  private_data                    = (inputtext_private_data_t *) xitk_xmalloc(sizeof(inputtext_private_data_t));
   
-  private_data->display       = it->display;
-  
-  private_data->iWidget       = mywidget;
-  private_data->text          = strdup((it->text != NULL)?it->text:"");
+  private_data->imlibdata         = it->imlibdata;
+  private_data->skin_element_name = (skin_element_name == NULL) ? NULL : strdup(it->skin_element_name);
 
-  private_data->fontname       = (it->fontname) ? strdup(it->fontname) : NULL;
+  private_data->iWidget           = mywidget;
+
+  private_data->text              = strdup((it->text != NULL)?it->text:"");
+
+  private_data->fontname          = strdup(fontname);
  
-  private_data->max_length    = it->max_length;
-  private_data->cursor_pos    = -1;
+  private_data->max_length        = it->max_length;
+  private_data->cursor_pos        = -1;
 
-  private_data->skin          = gui_load_image(it->imlibdata, it->skin_filename);
+  private_data->skin              = skin;
 
-  private_data->max_visible   = (private_data->skin->width/2) / 6;
-  private_data->disp_offset   = 0;
+  private_data->max_visible       = (private_data->skin->width/2);
+  private_data->disp_offset       = 0;
+  
+  private_data->callback          = it->callback;
+  private_data->userdata          = it->userdata;
+  
+  private_data->normal_color      = strdup(ncolor);
+  private_data->focused_color     = strdup(fcolor);
 
-  private_data->callback      = it->callback;
-  private_data->userdata      = it->userdata;
+  private_data->pos_in_pos        = 0;
+  private_data->pos[0]            = 0;
 
-  private_data->normal_color  = strdup(it->normal_color);
-  private_data->focused_color = strdup(it->focused_color);
+  mywidget->private_data          = private_data;
 
-  mywidget->private_data      = private_data;
+  mywidget->enable                = 1;
+  mywidget->running               = 1;
+  mywidget->visible               = 1;
+  mywidget->have_focus            = FOCUS_LOST;
+  mywidget->imlibdata             = private_data->imlibdata;
+  mywidget->x                     = x;
+  mywidget->y                     = y;
+  mywidget->width                 = private_data->skin->width/2;
+  mywidget->height                = private_data->skin->height;
+  mywidget->widget_type           = WIDGET_TYPE_INPUTTEXT;
 
-  mywidget->enable            = 1;
-  mywidget->running           = 1;
-  mywidget->visible           = 1;
-  mywidget->have_focus        = FOCUS_LOST;
-  mywidget->x                 = it->x;
-  mywidget->y                 = it->y;
+  mywidget->paint                 = paint_inputtext;
+  mywidget->notify_click          = notify_click_inputtext;
+  mywidget->notify_focus          = notify_focus_inputtext;
+  mywidget->notify_keyevent       = notify_keyevent_inputtext;
+  mywidget->notify_inside         = notify_inside;
+  mywidget->notify_change_skin    = (skin_element_name == NULL) ? NULL : notify_change_skin;
+  mywidget->notify_destroy        = notify_destroy;
+  mywidget->get_skin              = get_skin;
 
-  mywidget->width             = private_data->skin->width/2;
-  mywidget->height            = private_data->skin->height;
-  mywidget->widget_type       = WIDGET_TYPE_INPUTTEXT;
-
-  mywidget->paint             = paint_inputtext;
-  mywidget->notify_click      = notify_click_inputtext;
-  mywidget->notify_focus      = notify_focus_inputtext;
-  mywidget->notify_keyevent   = notify_keyevent_inputtext;
+  mywidget->tips_timeout          = 0;
+  mywidget->tips_string           = NULL;
 
   return mywidget;
 }
+
+
+xitk_widget_t *xitk_inputtext_create (xitk_skin_config_t *skonfig, xitk_inputtext_widget_t *it) {
+
+  XITK_CHECK_CONSTITENCY(it);
+
+  return _xitk_inputtext_create (skonfig, it,
+				 (xitk_skin_get_coord_x(skonfig, it->skin_element_name)),
+				 (xitk_skin_get_coord_y(skonfig, it->skin_element_name)),
+				 it->skin_element_name,
+				 (xitk_image_load_image(it->imlibdata, 
+							xitk_skin_get_skin_filename(skonfig, it->skin_element_name))),
+				 (xitk_skin_get_label_fontname(skonfig, it->skin_element_name)),
+				 (xitk_skin_get_label_color(skonfig, it->skin_element_name)),
+				 (xitk_skin_get_label_color_focus(skonfig, it->skin_element_name)));
+}
+
+/*
+ *
+ */
+xitk_widget_t *xitk_noskin_inputtext_create (xitk_inputtext_widget_t *it,
+					     int x, int y, int width, int height,
+					     char *ncolor, char *fcolor, char *fontname) {
+  xitk_image_t *i;
+
+  XITK_CHECK_CONSTITENCY(it);
+  i = xitk_image_create_image(it->imlibdata, width * 2, height);
+  draw_bevel_two_state(it->imlibdata, i);
+  
+
+  return _xitk_inputtext_create(NULL, it, x, y, NULL, i, fontname, ncolor, fcolor);
+}
+
