@@ -17,10 +17,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: widget.c,v 1.5 2001/07/23 23:24:45 f1rmb Exp $
+ * $Id: widget.c,v 1.17 2001/12/31 13:26:29 f1rmb Exp $
  *
  */
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -29,19 +28,21 @@
 #include <stdlib.h>
 #include <ctype.h>
 #include <string.h>
-#include <pthread.h>
 #include <errno.h>
 #include <inttypes.h>
 #include <unistd.h>
 
 #include <X11/Xlib.h>
+#include <X11/Intrinsic.h>
 
 #include "widget.h"
 #include "list.h"
+#include "_xitk.h"
+#include "tips.h"
 
 extern int errno;
 
-static gui_color_names_t gui_color_names[] = {
+static xitk_color_names_t xitk_color_names[] = {
   { 255,  250,  250,  "snow" },
   { 248,  248,  255,  "GhostWhite" },
   { 245,  245,  245,  "WhiteSmoke" },
@@ -701,208 +702,452 @@ static gui_color_names_t gui_color_names[] = {
 };
 
 /*
- *
+ * Alloc a memory area of size_t size.
  */
-void *gui_xmalloc(size_t size) {
-  void *ptrmalloc, *ptrmemset;
+void *xitk_xmalloc(size_t size) {
+  void *ptrmalloc;
+  
+  if(size <= 0) {
+    XITK_WARNING("%s(): size was <= 0.\n", __FUNCTION__);
+    return NULL;
+  }
 
   if((ptrmalloc = malloc(size)) == NULL) {
-    fprintf(stderr, "%s: malloc() failed: %s.\n",
-	    __FUNCTION__, strerror(errno));
+    XITK_WARNING("%s: malloc() failed: %s.\n", __FUNCTION__, strerror(errno));
     return NULL;
   }
 
-  if((ptrmemset = memset(ptrmalloc, 0, size)) == NULL) {
-    fprintf(stderr, "%s: memset() failed: %s.\n", 
-	    __FUNCTION__, strerror(errno));
-    return NULL;
-  }
+  memset(ptrmalloc, 0, size);
 
-  return ptrmemset;
+  return ptrmalloc;
 }
 
 /*
- *
+ * Call the notify_change_skin function, if exist, of each widget in
+ * the specifier widget list.
  */
-int paint_widget_list (widget_list_t *wl) {
-  widget_t *mywidget;
+void xitk_change_skins_widget_list(xitk_widget_list_t *wl, xitk_skin_config_t *skonfig) {
+  xitk_widget_t *mywidget;
 
-  mywidget = (widget_t *) gui_list_first_content (wl->l);
+  if(!wl) {
+    XITK_WARNING("%s(): widget list was NULL.\n", __FUNCTION__);
+    return;
+  }
+  mywidget = (xitk_widget_t *) xitk_list_first_content (wl->l);
+  while (mywidget && wl && wl->l && wl->win && wl->gc && skonfig) {
+    
+    if(mywidget->notify_change_skin)
+      (mywidget->notify_change_skin) (wl, mywidget, skonfig);
+    
+    mywidget = (xitk_widget_t *) xitk_list_next_content (wl->l); 
+  }
+}
+
+/*
+ * (re)paint the specified widget list.
+ */
+int xitk_paint_widget_list (xitk_widget_list_t *wl) {
+  xitk_widget_t *mywidget;
+
+  mywidget = (xitk_widget_t *) xitk_list_first_content (wl->l);
   while (mywidget && wl && wl->l && wl->win && wl->gc) {
-
+    
     if(mywidget->paint)
       (mywidget->paint) (mywidget, wl->win, wl->gc);
-
-    mywidget = (widget_t *) gui_list_next_content (wl->l); 
+    
+    mywidget = (xitk_widget_t *) xitk_list_next_content (wl->l); 
   }
   return 1;
 }
 
 /*
- *
+ * Return 1 if the mouse poiter is in the visible area of widget.
  */
-int is_inside_widget (widget_t *widget, int x, int y) {
-  return ((x >= widget->x) && (x <= (widget->x + widget->width))
-	  && (y >= widget->y) && (y <= (widget->y + widget->height)));
+int xitk_is_cursor_out_mask(Display *display, xitk_widget_t *w, Pixmap mask, int x, int y) {
+  XImage *xi;
+  Pixel p;
+  
+  if(!display) {
+    XITK_WARNING("%s(): display was unset.\n", __FUNCTION__);
+    return 0;
+  }
+  if(!w) {
+    XITK_WARNING("%s(): widget list was NULL.\n", __FUNCTION__);
+    return 0;
+  }
+
+  if(mask) {
+    int xx, yy;
+    
+    if((xx = (x - w->x)) == w->width) xx--;
+    if((yy = (y - w->y)) == w->height) yy--;
+    
+    XLOCK(display);
+    
+    xi = XGetImage(display, mask, xx, yy, 1, 1, AllPlanes, ZPixmap);
+    p = XGetPixel(xi, 0, 0);
+    XDestroyImage(xi);
+    
+    XUNLOCK(display);
+
+    return (int) p;
+  }
+  
+  return 1;
 }
 
 /*
- *
+ * Return 1 if mouse pointer is in widget area.
  */
-widget_t *get_widget_at (widget_list_t *wl, int x, int y) {
-  widget_t *mywidget;
+int xitk_is_inside_widget (xitk_widget_t *widget, int x, int y) {
+  int inside = 0;
+  
+  if(!widget) {
+    XITK_WARNING("%s(): widget was NULL.\n", __FUNCTION__);
+    return 0;
+  }
 
-  mywidget = (widget_t *) gui_list_first_content (wl->l);
+  if(((x >= widget->x) && (x <= (widget->x + widget->width))) && 
+     ((y >= widget->y) && (y <= (widget->y + widget->height)))) {
+    
+    inside = 1;
+    
+    if(widget->notify_inside) 
+      inside = widget->notify_inside(widget, x, y);
+
+  }
+
+  return inside;
+}
+
+/*
+ * Return widget present at specified coords.
+ */
+xitk_widget_t *xitk_get_widget_at (xitk_widget_list_t *wl, int x, int y) {
+  xitk_widget_t *mywidget;
+
+  if(!wl) {
+    XITK_WARNING("%s(): widget list was NULL.\n", __FUNCTION__);
+    return 0;
+  }
+
+  mywidget = (xitk_widget_t *) xitk_list_first_content (wl->l);
   while (mywidget) {
-    if (is_inside_widget (mywidget, x, y))
+    if (xitk_is_inside_widget (mywidget, x, y))
       return mywidget;
 
-    mywidget = (widget_t *) gui_list_next_content (wl->l);
+    mywidget = (xitk_widget_t *) xitk_list_next_content (wl->l);
   }
   return NULL;
 }
 
 /*
- *
+ * Call notify_focus function in right widget (the one who get, and
+ * the one who loose focus).
  */
-void motion_notify_widget_list (widget_list_t *wl,
+void xitk_motion_notify_widget_list (xitk_widget_list_t *wl,
 				int x, int y) {
   int bRepaint = 0;
-  widget_t *mywidget = get_widget_at (wl, x, y);
+  xitk_widget_t *mywidget;
+  
+  if(!wl) {
+    XITK_WARNING("%s(): widget list was NULL.\n", __FUNCTION__);
+    return;
+  }
+  
+  mywidget = xitk_get_widget_at (wl, x, y);
   
   if (mywidget != wl->focusedWidget) {
     if (wl->focusedWidget) {
-      if (wl->focusedWidget->notify_focus 
-	  && wl->focusedWidget->enable == WIDGET_ENABLE) {
+      
+      /* Kill (hide) tips */
+      xitk_tips_tips_kill(wl->focusedWidget);
+      
+      if (wl->focusedWidget->notify_focus && wl->focusedWidget->enable == WIDGET_ENABLE) {
 	bRepaint |= (wl->focusedWidget->notify_focus) (wl, wl->focusedWidget, FOCUS_LOST);
 	wl->focusedWidget->have_focus = FOCUS_LOST;
       }
+      if(wl->focusedWidget->paint)
+	(wl->focusedWidget->paint) (wl->focusedWidget, wl->win, wl->gc);
     }
     
     wl->focusedWidget = mywidget;
     
     if (mywidget) {
+      
+      if(mywidget->enable == WIDGET_ENABLE)
+	xitk_tips_create(mywidget, wl);
+      
       if (mywidget->notify_focus && mywidget->enable == WIDGET_ENABLE) {
 	bRepaint |= (mywidget->notify_focus) (wl, mywidget, FOCUS_RECEIVED);
 	mywidget->have_focus = FOCUS_RECEIVED;
       }
+
+      if(mywidget->paint)
+	(mywidget->paint) (mywidget, wl->win, wl->gc);
     }
+    
   }
-  
-  if (bRepaint)
-    paint_widget_list (wl);
 }
 
 /*
- *
+ * Call notify_click function, if exist, of widget at coords x/y.
  */
-int click_notify_widget_list (widget_list_t *wl, 
-			       int x, int y, int bUp) {
-  int bRepaint = 0;
-  widget_t *mywidget = get_widget_at (wl, x, y);
+int xitk_click_notify_widget_list (xitk_widget_list_t *wl, int x, int y, int bUp) {
+  int            bRepaint = 0;
+  xitk_widget_t  *mywidget;
 
-  motion_notify_widget_list (wl, x, y);
+  if(!wl) {
+    XITK_WARNING("%s(): widget list was NULL.\n", __FUNCTION__);
+    return 0;
+  }
+
+  mywidget = xitk_get_widget_at (wl, x, y);
+
+  xitk_motion_notify_widget_list (wl, x, y);
 
   if (!bUp) {
       
     wl->pressedWidget = mywidget;
     
     if (mywidget) {
-      if (mywidget->notify_click && mywidget->enable == WIDGET_ENABLE)
+
+      /* Kill (hide) tips */
+      xitk_tips_tips_kill(mywidget);
+
+      if (mywidget->notify_click && mywidget->enable == WIDGET_ENABLE && mywidget->running)
 	bRepaint |= (mywidget->notify_click) (wl, mywidget, LBUTTON_DOWN, x, y);
+
     }
   } else {
     if (wl->pressedWidget) {
       if (wl->pressedWidget->notify_click 
-	  && wl->pressedWidget->enable == WIDGET_ENABLE)
+	  && wl->pressedWidget->enable == WIDGET_ENABLE && wl->pressedWidget->running)
 	bRepaint |= (wl->pressedWidget->notify_click) (wl, wl->pressedWidget, LBUTTON_UP, x, y);
+
     }
   }
   
   return((bRepaint == 1));
-  /*
-  if (bRepaint && wl && wl->win && wl->gc) {
-    paint_widget_list (wl);
-    return 1;
-  } else
-    return 0;
-  */
 }
 
 /*
- *
+ * Call notify_keyevent, if exist, of specified plugin. This pass an X11 XEvent.
  */
-void widget_send_key_event(widget_list_t *wl, widget_t *w, XEvent *xev) {
+void xitk_send_key_event(xitk_widget_list_t *wl, xitk_widget_t *w, XEvent *xev) {
   
+  if(!w) {
+    XITK_WARNING("widget is NULL\n");
+    return;
+  }
   if(w->notify_keyevent)
     w->notify_keyevent(wl, w, xev);
 }
 
 /*
- *
+ * Return the focused widget.
  */
-int widget_get_width(widget_t *lb) {
- 
-  return lb->width;  
-}
+xitk_widget_t *xitk_get_focused_widget(xitk_widget_list_t *wl) {
 
-/*
- *
- */
-int widget_get_height(widget_t *lb) {
+  if(wl) {
+    return wl->focusedWidget;
+  }
   
-  return lb->height;  
+  XITK_WARNING("widget list is NULL\n");
+  return NULL;
 }
 
 /*
- *
+ * Return the pressed widget.
  */
-int widget_enabled(widget_t *w) {
+xitk_widget_t *xitk_get_pressed_widget(xitk_widget_list_t *wl) {
 
+  if(wl)
+    return wl->pressedWidget;
+  
+  XITK_WARNING("widget list is NULL\n");
+  return NULL;
+}
+
+/*
+ * Return widget width.
+ */
+int xitk_get_widget_width(xitk_widget_t *w) {
+ 
+  if(!w) {
+    XITK_WARNING("widget is NULL\n");
+    return -1;
+  }
+  return w->width;  
+}
+
+/*
+ * Return widget height.
+ */
+int xitk_get_widget_height(xitk_widget_t *w) {
+  
+  if(!w) {
+    XITK_WARNING("widget is NULL\n");
+    return -1;
+  }
+  return w->height;  
+}
+
+/*
+ * Set position of a widget.
+ */
+int xitk_set_widget_pos(xitk_widget_t *w, int x, int y) {
+
+  if(!w) {
+    XITK_WARNING("widget is NULL\n");
+    return 0;
+  }
+
+  w->x = x;
+  w->y = y;
+
+  return 1;
+}
+
+/*
+ * Get position of a widget.
+ */
+int xitk_get_widget_pos(xitk_widget_t *w, int *x, int *y) {
+
+  if(!w) {
+    XITK_WARNING("widget is NULL\n");
+    return 0;
+  }
+  *x = w->x;
+  *y = w->y;
+  return 1;
+}
+
+/*
+ * Return 1 if widget is enabled.
+ */
+int xitk_is_widget_enabled(xitk_widget_t *w) {
+
+  if(!w) {
+    XITK_WARNING("widget is NULL\n");
+    return 0;
+  }
   return (w->enable == WIDGET_ENABLE);
 }
 
 /*
- *
+ * Return 1 if widget have focus.
  */
-int widget_have_focus(widget_t *w) {
+int xitk_is_widget_focused(xitk_widget_t *w) {
 
+  if(!w) {
+    XITK_WARNING("widget is NULL\n");
+    return 0;
+  }
   return(w->have_focus == FOCUS_RECEIVED);
 }
 
 /*
- *
+ * Enable a widget.
  */
-void widget_enable(widget_t *w) {
+void xitk_enable_widget(xitk_widget_t *w) {
 
+  if(!w) {
+    XITK_WARNING("widget is NULL\n");
+    return;
+  }
   w->enable = WIDGET_ENABLE;
 }
 
 /*
- *
+ * Disable a widget.
  */
-void widget_disable(widget_t *w) {
+void xitk_disable_widget(xitk_widget_t *w) {
+
+  if(!w) {
+    XITK_WARNING("widget is NULL\n");
+    return;
+  }
 
   w->enable = ~WIDGET_ENABLE;
+
 }
 
 /*
  *
  */
-gui_color_names_t *gui_get_color_names(void) {
-
-  return gui_color_names;
+void xitk_free_widget(xitk_widget_t *w) {
+  
+  if(!w) {
+    XITK_WARNING("widget is NULL\n");
+    return;
+  }
+  
+  if(w->notify_destroy)
+    w->notify_destroy(w, NULL);
+  
+  XITK_FREE(w);
+  w = NULL;
+  
 }
 
 /*
- *
+ * Destroy a widget.
  */
-gui_color_names_t *gui_get_color_name(char *color) {
-  gui_color_names_t *cn = NULL;
-  gui_color_names_t *pcn = NULL;
+void xitk_destroy_widget(xitk_widget_list_t *wl, xitk_widget_t *w) {
 
-  if(color == NULL)
+  if(!w) {
+    XITK_WARNING("widget is NULL\n");
+    return;
+  }
+
+  xitk_hide_widget(wl, w);
+  xitk_stop_widget(w);
+  xitk_disable_widget(w);
+
+  xitk_free_widget(w);
+}
+
+/*
+ * Show widgets from widget list.
+ */
+void xitk_destroy_widgets(xitk_widget_list_t *wl) {
+  xitk_widget_t *mywidget;
+  
+  if(!wl) {
+    XITK_WARNING("%s(): widget list was NULL.\n", __FUNCTION__);
+    return;
+  }
+  
+  mywidget = (xitk_widget_t *) xitk_list_last_content (wl->l);
+
+  while(mywidget) {
+    
+    xitk_destroy_widget(wl, mywidget);
+    
+    mywidget = (xitk_widget_t *) xitk_list_prev_content (wl->l);
+  }
+}
+
+/*
+ * Return the struct of color names/values.
+ */
+xitk_color_names_t *gui_get_color_names(void) {
+
+  return xitk_color_names;
+}
+
+/*
+ * Return a xitk_color_name_t type from a string color.
+ */
+xitk_color_names_t *xitk_get_color_name(char *color) {
+  xitk_color_names_t *cn = NULL;
+  xitk_color_names_t *pcn = NULL;
+
+  if(color == NULL) {
+    XITK_WARNING("%s(): color was NULL.\n", __FUNCTION__);
     return NULL;
+  }
 
   /* Hexa X triplet */
   if((strncasecmp(color, "#", 1) <= 0) && (strlen(color) == 7)) {
@@ -922,25 +1167,25 @@ gui_color_names_t *gui_get_color_name(char *color) {
     }
     lowercolorname = p;
 
-    cn = (gui_color_names_t *) gui_xmalloc(sizeof(gui_color_names_t));
+    cn = (xitk_color_names_t *) xitk_xmalloc(sizeof(xitk_color_names_t));
     
     if((sscanf(lowercolorname, 
 	       "%2x%2x%2x", &cn->red, &cn->green, &cn->blue)) != 3) {
-      fprintf(stderr, "sscanf() failed: %s\n", strerror(errno));
+      XITK_WARNING("sscanf() failed: %s\n", strerror(errno));
       cn->red = cn->green = cn->blue = 0;
     }
 
     cn->colorname = strdup(color);
     
-    free(p);
+    XITK_FREE(p);
 
     return cn;
   } 
   else {
-    for(pcn = gui_color_names; pcn->colorname != NULL; pcn++) {
+    for(pcn = xitk_color_names; pcn->colorname != NULL; pcn++) {
       if(!strncasecmp(pcn->colorname, color, strlen(pcn->colorname))) {
 
-	cn = (gui_color_names_t *) gui_xmalloc(sizeof(gui_color_names_t));
+	cn = (xitk_color_names_t *) xitk_xmalloc(sizeof(xitk_color_names_t));
 
 	cn->red       = pcn->red;
 	cn->green     = pcn->green;
@@ -956,49 +1201,302 @@ gui_color_names_t *gui_get_color_name(char *color) {
 }
 
 /*
- *
+ * Stop a widget.
  */
-void widget_stop_widgets(widget_list_t *wl) {
-  widget_t *mywidget;
+void xitk_stop_widget(xitk_widget_t *w) {
+
+  if(!w) {
+    XITK_WARNING("widget is NULL\n");
+    return;
+  }
+
+  w->running = 0;
   
-  mywidget = (widget_t *) gui_list_first_content (wl->l);
+}
+
+/*
+ * (Re)Start a widget.
+ */
+void xitk_start_widget(xitk_widget_t *w) {
+
+  if(!w) {
+    XITK_WARNING("widget is NULL\n");
+    return;
+  }
+
+  w->running = 1;
+  
+}
+
+/*
+ * Stop a widgets from widget list.
+ */
+void xitk_stop_widgets(xitk_widget_list_t *wl) {
+  xitk_widget_t *mywidget;
+  
+  if(!wl) {
+    XITK_WARNING("%s(): widget list was NULL.\n", __FUNCTION__);
+    return;
+  }
+    
+  mywidget = (xitk_widget_t *) xitk_list_first_content (wl->l);
 
   while(mywidget) {
     
-    mywidget->running = 0;
+    xitk_stop_widget(mywidget);
     
-    mywidget = (widget_t *) gui_list_next_content (wl->l);
+    mywidget = (xitk_widget_t *) xitk_list_next_content (wl->l);
+  }
+}
+
+/*
+ * Show a widget.
+ */
+void xitk_show_widget(xitk_widget_list_t *wl, xitk_widget_t *w) {
+
+  if(!w) {
+    XITK_WARNING("widget is NULL\n");
+    return;
+  }
+
+  w->visible = 1;
+  
+  if(w->paint)
+    w->paint(w, wl->win, wl->gc);
+}
+
+/*
+ * Show widgets from widget list.
+ */
+void xitk_show_widgets(xitk_widget_list_t *wl) {
+  xitk_widget_t *mywidget;
+  
+  if(!wl) {
+    XITK_WARNING("%s(): widget list was NULL.\n", __FUNCTION__);
+    return;
+  }
+    
+  mywidget = (xitk_widget_t *) xitk_list_first_content (wl->l);
+
+  while(mywidget) {
+    
+    xitk_show_widget(wl, mywidget);
+    
+    mywidget = (xitk_widget_t *) xitk_list_next_content (wl->l);
+  }
+}
+
+/*
+ * Hide a widget.
+ */
+void xitk_hide_widget(xitk_widget_list_t *wl, xitk_widget_t *w) {
+
+  if(!w) {
+    XITK_WARNING("widget is NULL\n");
+    return;
+  }
+
+  w->visible = 0;
+
+  if(w->paint)
+    w->paint(w, wl->win, wl->gc);
+  
+}
+
+/*
+ * Hide widgets from widget list..
+ */
+void xitk_hide_widgets(xitk_widget_list_t *wl) {
+  xitk_widget_t *mywidget;
+  
+  if(!wl) {
+    XITK_WARNING("%s(): widget list was NULL.\n", __FUNCTION__);
+    return;
+  }
+    
+  mywidget = (xitk_widget_t *) xitk_list_first_content (wl->l);
+
+  while(mywidget) {
+    
+    xitk_hide_widget(wl, mywidget);
+    
+    mywidget = (xitk_widget_t *) xitk_list_next_content (wl->l);
   }
 }
 
 /*
  *
  */
-void widget_show_widgets(widget_list_t *wl) {
-  widget_t *mywidget;
+xitk_image_t *xitk_get_widget_foreground_skin(xitk_widget_t *w) {
+
+  if(!w) {
+    XITK_WARNING("widget is NULL\n");
+    return NULL;
+  }
+
+  if(w->get_skin)
+    return w->get_skin(w, FOREGROUND_SKIN);
+
+  return NULL;
+}
+
+/*
+ *
+ */
+xitk_image_t *xitk_get_widget_background_skin(xitk_widget_t *w) {
+
+  if(!w) {
+    XITK_WARNING("widget is NULL\n");
+    return NULL;
+  }
+
+  if(w->get_skin)
+    return w->get_skin(w, BACKGROUND_SKIN);
+
+  return NULL;
+}
+
+/*
+ *
+ */
+void xitk_set_widget_tips(xitk_widget_t *w, char *str) {
+
+  if(!w || !str) {
+    XITK_WARNING("widget is NULL\n");
+    return;
+  }
+
+  xitk_tips_set_tips(w, str);
+}
+
+/*
+ *
+ */
+void xitk_set_widget_tips_default(xitk_widget_t *w, char *str) {
+
+  if(!w || !str) {
+    XITK_WARNING("widget is NULL\n");
+    return;
+  }
+
+  xitk_tips_set_tips(w, str);
+  xitk_tips_set_timeout(w, TIPS_TIMEOUT);
+}
+
+/*
+ *
+ */
+void xitk_set_widget_tips_and_timeout(xitk_widget_t *w, char *str, unsigned int timeout) {
+
+  if(!w || !str) {
+    XITK_WARNING("widget is NULL\n");
+    return;
+  }
+
+  xitk_tips_set_tips(w, str);
+  xitk_tips_set_timeout(w, timeout);
+}
+
+/*
+ *
+ */
+void xitk_disable_widget_tips(xitk_widget_t *w) {
   
-  mywidget = (widget_t *) gui_list_first_content (wl->l);
+  if(!w) {
+    XITK_WARNING("widget is NULL\n");
+    return;
+  }
+  
+  xitk_tips_set_timeout(w, 0);
+}
+
+/*
+ *
+ */
+void xitk_enable_widget_tips(xitk_widget_t *w) {
+  
+  if(!w) {
+    XITK_WARNING("widget is NULL\n");
+    return;
+  }
+  
+  xitk_tips_set_timeout(w, TIPS_TIMEOUT);
+}
+
+/*
+ *
+ */
+void xitk_disable_widgets_tips(xitk_widget_list_t *wl) {
+  xitk_widget_t *mywidget;
+  
+  if(!wl) {
+    XITK_WARNING("%s(): widget list was NULL.\n", __FUNCTION__);
+    return;
+  }
+  
+  mywidget = (xitk_widget_t *) xitk_list_first_content (wl->l);
 
   while(mywidget) {
     
-    mywidget->visible = 1;
+    xitk_disable_widget_tips(mywidget);
     
-    mywidget = (widget_t *) gui_list_next_content (wl->l);
+    mywidget = (xitk_widget_t *) xitk_list_next_content (wl->l);
   }
 }
 
 /*
  *
  */
-void widget_hide_widgets(widget_list_t *wl) {
-  widget_t *mywidget;
+void xitk_enable_widgets_tips(xitk_widget_list_t *wl) {
+  xitk_widget_t *mywidget;
   
-  mywidget = (widget_t *) gui_list_first_content (wl->l);
+  if(!wl) {
+    XITK_WARNING("%s(): widget list was NULL.\n", __FUNCTION__);
+    return;
+  }
+  
+  mywidget = (xitk_widget_t *) xitk_list_first_content (wl->l);
 
   while(mywidget) {
     
-    mywidget->visible = 0;
+    if(mywidget->tips_string)
+      xitk_enable_widget_tips(mywidget);
     
-    mywidget = (widget_t *) gui_list_next_content (wl->l);
+    mywidget = (xitk_widget_t *) xitk_list_next_content (wl->l);
   }
 }
+
+/*
+ *
+ */
+void xitk_set_widgets_tips_timeout(xitk_widget_list_t *wl, unsigned long timeout) {
+  xitk_widget_t *mywidget;
+  
+  if(!wl) {
+    XITK_WARNING("%s(): widget list was NULL.\n", __FUNCTION__);
+    return;
+  }
+
+  mywidget = (xitk_widget_t *) xitk_list_first_content (wl->l);
+
+  while(mywidget) {
+    
+    if(mywidget->tips_string)
+      xitk_tips_set_timeout(mywidget, timeout);
+    
+    mywidget = (xitk_widget_t *) xitk_list_next_content (wl->l);
+  }
+}
+
+/*
+ *
+ */
+void xitk_set_widget_tips_timeout(xitk_widget_t *w, unsigned long timeout) {
+
+  if(!w) {
+    XITK_WARNING("widget is NULL\n");
+    return;
+  }
+
+  xitk_tips_set_timeout(w, timeout);
+}
