@@ -17,12 +17,12 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: widget_types.h,v 1.4 2001/07/15 00:49:22 f1rmb Exp $
+ * $Id: widget_types.h,v 1.8 2001/11/26 10:21:26 f1rmb Exp $
  *
  */
 
-#ifndef HAVE_WIDGET_TYPES_H
-#define HAVE_WIDGET_TYPES_H
+#ifndef HAVE_XITK_WIDGET_TYPES_H
+#define HAVE_XITK_WIDGET_TYPES_H
 
 #define WIDGET_TYPE_GROUP         0xFFFF8000
 
@@ -36,6 +36,9 @@
 #define WIDGET_TYPE_FILEBROWSER   0x00000080
 #define WIDGET_TYPE_MRLBROWSER    0x00000100
 #define WIDGET_TYPE_INPUTTEXT     0x00000200
+#define WIDGET_TYPE_COMBO         0x00000400
+#define WIDGET_TYPE_TABS          0x00000800
+#define WIDGET_TYPE_INTBOX        0x00001000
 
 #endif
 
