@@ -17,11 +17,10 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xitk.c,v 1.19 2001/09/06 14:22:35 jkeil Exp $
+ * $Id: xitk.c,v 1.29 2002/01/09 00:29:56 f1rmb Exp $
  *
  */
-
-#define _XITK_C_ 1
+#define _XITK_C_
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -32,11 +31,17 @@
 #include <stdlib.h>
 #include <string.h>
 #include <pthread.h>
+#include <pwd.h>
 #include <errno.h>
 #include <inttypes.h>
 #include <unistd.h>
 #include <signal.h>
+#include <time.h>
 #include <setjmp.h>
+#include <sys/types.h>
+#include <sys/wait.h>
+
+#include <locale.h>
 
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
@@ -44,10 +49,12 @@
 #include "widget.h"
 #include "list.h"
 #include "dnd.h"
+#include "_config.h"
 
 #include "_xitk.h"
 
-extern int              errno;
+extern char **environ;
+extern int errno;
 #undef TRACE_LOCKS
 
 #ifdef TRACE_LOCKS
@@ -75,58 +82,129 @@ static int ml = 0;
 #define MUTUNLOCK() { pthread_mutex_unlock(&gXitk->mutex); }
 #endif
 
-typedef void (*widget_cb_event_t)(XEvent *event, void *user_data);
-typedef void (*widget_cb_newpos_t)(int, int, int, int);
+typedef void (*widget_event_callback_t)(XEvent *event, void *user_data);
+typedef void (*widget_newpos_callback_t)(int, int, int, int);
+typedef void (*xitk_signal_callback_t)(int, void *);
 
 typedef struct {
-  Window              window;
-  Atom                XA_XITK;
+  Window                      window;
+  Atom                        XA_XITK;
 
-  gui_move_t          move;
+  xitk_move_t                  move;
 
   struct {
-    int               x;
-    int               y;
+    int                         x;
+    int                         y;
   } old_pos;
 
   struct {
-    int               x;
-    int               y;
+    int                         x;
+    int                         y;
   } new_pos;
 
-  int                width;
-  int                height;
+  int                         width;
+  int                         height;
 
-  XEvent             *old_event;
-  widget_list_t      *widget_list;
-  char               *name;
-  widget_cb_event_t   xevent_callback;
-  widget_cb_newpos_t  newpos_callback;
-  widgetkey_t         key;
-  xitk_dnd_t         *xdnd;
-  void               *user_data;
+  XEvent                     *old_event;
+  xitk_widget_list_t         *widget_list;
+  char                       *name;
+  widget_event_callback_t     xevent_callback;
+  widget_newpos_callback_t    newpos_callback;
+  xitk_register_key_t         key;
+  xitk_dnd_t                 *xdnd;
+  void                       *user_data;
 } __gfx_t;
 
 typedef struct {
-  Display            *display;
-  gui_list_t         *list;
-  gui_list_t         *gfx;
-  pthread_mutex_t     mutex;
-  int                 running;
-  widgetkey_t         key;
+  Display                    *display;
+  xitk_list_t                *list;
+  xitk_list_t                *gfx;
+  pthread_mutex_t             mutex;
+  int                         running;
+  xitk_register_key_t         key;
+  xitk_config_t              *config;
+  xitk_signal_callback_t      sig_callback;
+  void                       *sig_data;
 } __xitk_t;
 
-static __xitk_t  *gXitk;
-static pid_t      xitk_pid;
-static sigjmp_buf kill_jmp;
+static __xitk_t    *gXitk;
+static pid_t        xitk_pid;
+static sigjmp_buf   kill_jmp;
 
 
 void widget_stop(void);
 
+/*
+ * Execute a shell command.
+ */
+int xitk_system(int dont_run_as_root, char *command) {
+  int pid, status;
+  
+  /* 
+   * Don't permit run as root
+   */
+  if(dont_run_as_root) {
+    if(getuid() == 0)
+      return -1;
+  }
+  
+  if(command == 0)
+    return 1;
+  
+  pid = fork();
+  
+  if(pid == -1)
+    return -1;
+  
+  if(pid == 0) {
+    char *argv[4];
+    argv[0] = "sh";
+    argv[1] = "-c";
+    argv[2] = command;
+    argv[3] = 0;
+    execve("/bin/sh", argv, environ);
+    exit(127);
+  }
+  
+  do {
+    if(waitpid(pid, &status, 0) == -1) {
+      if (errno != EINTR)
+	return -1;
+    } 
+    else {
+      return WEXITSTATUS(status);
+    }
+  } while(1);
+  
+  return -1;
+}
 
+/*
+ * A thread-safe usecond sleep
+ */
+void xitk_usec_sleep(unsigned usec) {
+#if HAVE_NANOSLEEP
+  /* nanosleep is prefered on solaris, because it's mt-safe */
+  struct timespec ts;
+  
+  ts.tv_sec =   usec / 1000000;
+  ts.tv_nsec = (usec % 1000000) * 1000;
+  nanosleep(&ts, NULL);
+#else
+  usleep(usec);
+#endif
+}
+
+/*
+ *
+ */
 static void xitk_signal_handler(int sig) {
   pid_t cur_pid = getppid();
 
+  /* First, call registered handler */
+  if(gXitk->sig_callback)
+    gXitk->sig_callback(sig, gXitk->sig_data);
+  
   switch (sig) {
 
   case SIGINT:
@@ -135,7 +213,7 @@ static void xitk_signal_handler(int sig) {
     if(cur_pid == xitk_pid) {
       __gfx_t   *fx;
       
-      fprintf(stderr, "XITK killed with signal %d\n", sig);
+      XITK_WARNING("XITK killed with signal %d\n", sig);
 
       MUTLOCK();
       gXitk->running = 0;
@@ -143,7 +221,7 @@ static void xitk_signal_handler(int sig) {
       /* 
        * Tada.... and now Ladies and Gentlemen, the dirty hack
        */
-      fx = (__gfx_t *) gui_list_first_content(gXitk->gfx);
+      fx = (__gfx_t *) xitk_list_first_content(gXitk->gfx);
 
       while(fx) {
 	if(fx->window) {
@@ -161,8 +239,7 @@ static void xitk_signal_handler(int sig) {
 	  
 	  XLOCK(gXitk->display);
 	  if (!XSendEvent (gXitk->display, fx->window, True, 0L, &event)) {
-	    fprintf(stderr, "XSendEvent(display, 0x%x ...) failed.\n",
-		    (unsigned int) fx->window);
+	    XITK_WARNING("XSendEvent(display, 0x%x ...) failed.\n", (unsigned int) fx->window);
 	  }
 	  XUNLOCK(gXitk->display);
 
@@ -170,7 +247,7 @@ static void xitk_signal_handler(int sig) {
 	  siglongjmp(kill_jmp, 1);
 	  return;
 	}
-	fx = (__gfx_t *) gui_list_next_content(gXitk->gfx);
+	fx = (__gfx_t *) xitk_list_next_content(gXitk->gfx);
       }
       MUTUNLOCK();
     }
@@ -179,16 +256,16 @@ static void xitk_signal_handler(int sig) {
 }
 /*
  * Create a new widget_list, store the pointer in private
- * list of widget_list_t, then return the widget_list pointer.
+ * list of xitk_widget_list_t, then return the widget_list pointer.
  */
-widget_list_t *widget_list_new (void) {
-  widget_list_t *l;
+xitk_widget_list_t *xitk_widget_list_new (void) {
+  xitk_widget_list_t *l;
 
   MUTLOCK();
 
-  l = (widget_list_t *) gui_xmalloc(sizeof(widget_list_t));
+  l = (xitk_widget_list_t *) xitk_xmalloc(sizeof(xitk_widget_list_t));
 
-  gui_list_append_content(gXitk->list, l);
+  xitk_list_append_content(gXitk->list, l);
 
   MUTUNLOCK();
 
@@ -200,12 +277,12 @@ widget_list_t *widget_list_new (void) {
  * at widget_register_event_handler() time. It also remade it
  * DND aware, only if DND stuff was initialized at register time too.
  */
-void widget_change_window_for_event_handler (widgetkey_t key, Window window) {
+void xitk_change_window_for_event_handler (xitk_register_key_t key, Window window) {
   __gfx_t  *fx;
   
   MUTLOCK();
       
-  fx = (__gfx_t *) gui_list_first_content(gXitk->gfx);
+  fx = (__gfx_t *) xitk_list_first_content(gXitk->gfx);
   
   while(fx) {
 
@@ -216,31 +293,42 @@ void widget_change_window_for_event_handler (widgetkey_t key, Window window) {
       fx->window = window;
 
       if(fx->xdnd && (window != None))
-	dnd_make_window_aware(fx->xdnd, window);
+	xitk_make_window_dnd_aware(fx->xdnd, window);
       
       XUNLOCK(gXitk->display);
       MUTUNLOCK();
       return;
     }
-    fx = (__gfx_t *) gui_list_next_content(gXitk->gfx);
+    fx = (__gfx_t *) xitk_list_next_content(gXitk->gfx);
   }
 
   MUTUNLOCK();
 }
+
+/*
+ * Register a callback function called when a signal happen.
+ */
+void xitk_register_signal_handler(xitk_signal_callback_t sigcb, void *user_data) {
+  if(sigcb) {
+    gXitk->sig_callback = sigcb;
+    gXitk->sig_data     = user_data;
+  }
+}
+
 /*
  * Register a window, with his own event handler, callback
  * for DND events, and widget list.
  */
-widgetkey_t widget_register_event_handler(char *name, Window window,
-					  widget_cb_event_t cb,
-					  widget_cb_newpos_t pos_cb,
-					  xitk_dnd_callback_t dnd_cb,
-					  widget_list_t *wl, void *user_data) {
+xitk_register_key_t xitk_register_event_handler(char *name, Window window,
+						widget_event_callback_t cb,
+						widget_newpos_callback_t pos_cb,
+						xitk_dnd_callback_t dnd_cb,
+						xitk_widget_list_t *wl, void *user_data) {
   __gfx_t   *fx;
 
   //  printf("%s()\n", __FUNCTION__);
 
-  fx = (__gfx_t *) gui_xmalloc(sizeof(__gfx_t));
+  fx = (__gfx_t *) xitk_xmalloc(sizeof(__gfx_t));
 
   fx->name = (name != NULL) ? name : "NO_SET";
 
@@ -269,11 +357,11 @@ widgetkey_t widget_register_event_handler(char *name, Window window,
 
   if(cb)
     fx->xevent_callback = cb;
+#if 0
   else {
-    fprintf(stderr, "%s()@%d: Callback should be non NULL\n",
-	    __FUNCTION__, __LINE__);
-    exit(1);
+    XITK_DIE("%s()@%d: Callback should be non NULL\n", __FUNCTION__, __LINE__);
   }
+#endif
 
   if(pos_cb && window)
     fx->newpos_callback = pos_cb;
@@ -286,11 +374,11 @@ widgetkey_t widget_register_event_handler(char *name, Window window,
     fx->widget_list = NULL;
 
   if(dnd_cb && (window != None)) {
-    fx->xdnd = (xitk_dnd_t *) gui_xmalloc(sizeof(xitk_dnd_t));
+    fx->xdnd = (xitk_dnd_t *) xitk_xmalloc(sizeof(xitk_dnd_t));
     
-    dnd_init_dnd(gXitk->display, fx->xdnd);
-    dnd_set_callback(fx->xdnd, dnd_cb);
-    dnd_make_window_aware(fx->xdnd, fx->window);
+    xitk_init_dnd(gXitk->display, fx->xdnd);
+    xitk_set_dnd_callback(fx->xdnd, dnd_cb);
+    xitk_make_window_dnd_aware(fx->xdnd, fx->window);
   }
   else
   fx->xdnd = NULL;
@@ -311,7 +399,7 @@ widgetkey_t widget_register_event_handler(char *name, Window window,
 
   MUTLOCK();
 
-  gui_list_append_content(gXitk->gfx, fx);
+  xitk_list_append_content(gXitk->gfx, fx);
 
   MUTUNLOCK();
 
@@ -322,26 +410,26 @@ widgetkey_t widget_register_event_handler(char *name, Window window,
  * Remove from the list the window/event_handler
  * specified by the key.
  */
-void widget_unregister_event_handler(widgetkey_t *key) {
+void xitk_unregister_event_handler(xitk_register_key_t *key) {
   __gfx_t  *fx;
 
   //  printf("%s()\n", __FUNCTION__);
 
   MUTLOCK();
 
-  fx = (__gfx_t *) gui_list_first_content(gXitk->gfx);
+  fx = (__gfx_t *) xitk_list_first_content(gXitk->gfx);
   
   
   while(fx) {
 
     if(fx->key == *key) {
-      gui_list_delete_current(gXitk->gfx); 
+      xitk_list_delete_current(gXitk->gfx); 
 
       *key = 0; 
       MUTUNLOCK();
       return;
     }
-    fx = (__gfx_t *) gui_list_next_content(gXitk->gfx);
+    fx = (__gfx_t *) xitk_list_next_content(gXitk->gfx);
 
   }
   
@@ -352,12 +440,12 @@ void widget_unregister_event_handler(widgetkey_t *key) {
 /*
  * Copy window information matching with key in passed window_info_t struct.
  */
-int widget_get_window_info(widgetkey_t key, window_info_t *winf) {
+int xitk_get_window_info(xitk_register_key_t key, window_info_t *winf) {
   __gfx_t  *fx;
   
   MUTLOCK();
 
-  fx = (__gfx_t *) gui_list_first_content(gXitk->gfx);
+  fx = (__gfx_t *) xitk_list_first_content(gXitk->gfx);
     
   while(fx) {
 
@@ -377,7 +465,7 @@ int widget_get_window_info(widgetkey_t key, window_info_t *winf) {
       return 1;
 
     }
-    fx = (__gfx_t *) gui_list_next_content(gXitk->gfx);
+    fx = (__gfx_t *) xitk_list_next_content(gXitk->gfx);
 
   }
 
@@ -389,24 +477,25 @@ int widget_get_window_info(widgetkey_t key, window_info_t *winf) {
  * handled, then if a event handler callback was passed
  * at register time, it will be called.
  */
-static void widget_xevent_notify(XEvent *event) {
+void xitk_xevent_notify(XEvent *event) {
   __gfx_t  *fx;
 
     
-  fx = (__gfx_t *) gui_list_first_content(gXitk->gfx);
+  fx = (__gfx_t *) xitk_list_first_content(gXitk->gfx);
 
   while(fx) {
 
     if(fx->window != None) {
-
+      
       //      printf("event %d\n", event->type);
 
       if(fx->window == event->xany.window) {
 	
 	switch(event->type) {
+
 	case Expose:
 	  if (fx->widget_list && (event->xexpose.count == 0)) {
-	    paint_widget_list (fx->widget_list);
+	    xitk_paint_widget_list (fx->widget_list);
 	  }
 	  /*
 	  else {
@@ -458,19 +547,38 @@ static void widget_xevent_notify(XEvent *event) {
 	  }
 	  else {
 	    if(fx->widget_list)
-	      motion_notify_widget_list (fx->widget_list,
-					 event->xbutton.x, event->xbutton.y);
+	      xitk_motion_notify_widget_list (fx->widget_list,
+					      event->xbutton.x, event->xbutton.y);
 	  }
 	}
 	break;
 	
-	case ButtonPress:
+	case ButtonPress: {
+	  XWindowAttributes   wattr;
+	  Status              status;
+	  Window              focused_window;
+	  int                 revert;
+	  
+	  XLOCK(gXitk->display);
+	  status = XGetWindowAttributes(gXitk->display, fx->window, &wattr);
+	  XGetInputFocus(gXitk->display, &focused_window, &revert);
+
+	  /* 
+	   * Give focus(and raise) to a window after a click, and 
+	   * only if window isn't the current focused one.
+	   */
+	  if((status != BadDrawable) && (status != BadWindow) 
+	     && (wattr.map_state == IsViewable) && (focused_window != fx->window)) {
+	    XRaiseWindow(gXitk->display, fx->window);
+	    XSetInputFocus(gXitk->display, fx->window, RevertToParent, CurrentTime);
+	  }
+	  XUNLOCK(gXitk->display);
 	  
 	  if(fx->widget_list) {
 
-	    fx->move.enabled = !click_notify_widget_list (fx->widget_list, 
-							  event->xbutton.x, 
-							  event->xbutton.y, 0);
+	    fx->move.enabled = !xitk_click_notify_widget_list (fx->widget_list, 
+							       event->xbutton.x, 
+							       event->xbutton.y, 0);
 	    if (fx->move.enabled) {
 	      XWindowAttributes wattr;
 	      Status            err;
@@ -491,7 +599,8 @@ static void widget_xevent_notify(XEvent *event) {
 
 	    }
 	  }
-	  break;
+	}
+	break;
 	  
 	case ButtonRelease:
 	  
@@ -504,8 +613,8 @@ static void widget_xevent_notify(XEvent *event) {
 	  }
 	  
 	  if(fx->widget_list) {
-	    click_notify_widget_list (fx->widget_list, event->xbutton.x, 
-				      event->xbutton.y, 1);
+	    xitk_click_notify_widget_list (fx->widget_list, event->xbutton.x, 
+					   event->xbutton.y, 1);
 	  }
 	  break;
 	  
@@ -531,7 +640,7 @@ static void widget_xevent_notify(XEvent *event) {
 
 	case ClientMessage:
 	  if(fx->xdnd)
-	    dnd_process_client_message(fx->xdnd, event);
+	    xitk_process_client_dnd_message(fx->xdnd, event);
 	  break;
 	}
 
@@ -550,25 +659,32 @@ static void widget_xevent_notify(XEvent *event) {
       
     }
     
-    fx = (__gfx_t *) gui_list_next_content(gXitk->gfx);
+    fx = (__gfx_t *) xitk_list_next_content(gXitk->gfx);
   }
 }
 
 /*
  * Initiatization of widget internals.
  */
-void widget_init(Display *display) {
+void xitk_init(Display *display) {
 
 
   xitk_pid = getppid();
 
-  gXitk = (__xitk_t *) gui_xmalloc(sizeof(__xitk_t));
+#ifdef ENABLE_NLS
+  bindtextdomain("xitk", XITK_LOCALE);
+#endif  
 
-  gXitk->list    = gui_list_new();
-  gXitk->gfx     = gui_list_new();
-  gXitk->display = display;
-  gXitk->key     = 0;
+  gXitk = (__xitk_t *) xitk_xmalloc(sizeof(__xitk_t));
 
+  gXitk->list         = xitk_list_new();
+  gXitk->gfx          = xitk_list_new();
+  gXitk->display      = display;
+  gXitk->key          = 0;
+  gXitk->sig_callback = NULL;
+  gXitk->sig_data     = NULL;
+  gXitk->config       = xitk_config_init();
+  
   pthread_mutex_init (&gXitk->mutex, NULL);
 
 }
@@ -577,41 +693,59 @@ void widget_init(Display *display) {
  * Start widget event handling.
  * It will block till widget_stop() call
  */
-void widget_run(void) {
+void xitk_run(void) {
   XEvent            myevent;
   struct sigaction  action;
   __gfx_t          *fx;
 
+  action.sa_handler = xitk_signal_handler;
+  sigemptyset(&(action.sa_mask));
+  action.sa_flags = 0;
+  if(sigaction(SIGHUP, &action, NULL) != 0) {
+    XITK_WARNING("sigaction(SIGHUP) failed: %s\n", strerror(errno));
+  }
+  action.sa_handler = xitk_signal_handler;
+  sigemptyset(&(action.sa_mask));
+  action.sa_flags = 0;
+  if(sigaction(SIGUSR1, &action, NULL) != 0) {
+    XITK_WARNING("sigaction(SIGUSR1) failed: %s\n", strerror(errno));
+  }
+  action.sa_handler = xitk_signal_handler;
+  sigemptyset(&(action.sa_mask));
+  action.sa_flags = 0;
+  if(sigaction(SIGUSR2, &action, NULL) != 0) {
+    XITK_WARNING("sigaction(SIGUSR2) failed: %s\n", strerror(errno));
+  }
   action.sa_handler = xitk_signal_handler;
   sigemptyset(&(action.sa_mask));
   action.sa_flags = 0;
   if(sigaction(SIGINT, &action, NULL) != 0) {
-    fprintf(stderr, "sigaction(SIGINT) failed: %s\n", strerror(errno));
+    XITK_WARNING("sigaction(SIGINT) failed: %s\n", strerror(errno));
   }
   action.sa_handler = xitk_signal_handler;
   sigemptyset(&(action.sa_mask));
   action.sa_flags = 0;
   if(sigaction(SIGTERM, &action, NULL) != 0) {
-    fprintf(stderr, "sigaction(SIGTERM) failed: %s\n", strerror(errno));
+    XITK_WARNING("sigaction(SIGTERM) failed: %s\n", strerror(errno));
   }
   action.sa_handler = xitk_signal_handler;
   sigemptyset(&(action.sa_mask));
   action.sa_flags = 0;
   if(sigaction(SIGQUIT, &action, NULL) != 0) {
-    fprintf(stderr, "sigaction(SIGQUIT) failed: %s\n", strerror(errno));
+    XITK_WARNING("sigaction(SIGQUIT) failed: %s\n", strerror(errno));
   }
-
+  
   gXitk->running = 1;
-
+  
   /*
    * Force to repain the widget list if it exist
    */
   MUTLOCK();
-
-  fx = (__gfx_t *) gui_list_first_content(gXitk->gfx);
+  
+  fx = (__gfx_t *) xitk_list_first_content(gXitk->gfx);
   
   while(fx) {
-
+    
     if(fx->window != None && fx->widget_list) {
       XEvent xexp;
       
@@ -624,12 +758,11 @@ void widget_run(void) {
       
       XLOCK(gXitk->display);
       if(!XSendEvent(gXitk->display, fx->window, False, ExposureMask, &xexp)) {
-	fprintf(stderr, "XSendEvent(display, 0x%x ...) failed.\n",
-		(unsigned int) fx->window);
+	XITK_WARNING("XSendEvent(display, 0x%x ...) failed.\n", (unsigned int) fx->window);
       }
       XUNLOCK(gXitk->display);
     }
-    fx = (__gfx_t *) gui_list_next_content(gXitk->gfx);
+    fx = (__gfx_t *) xitk_list_next_content(gXitk->gfx);
   }
 
   MUTUNLOCK();
@@ -648,11 +781,11 @@ void widget_run(void) {
     
       XNextEvent (gXitk->display, &myevent) ;
       /* XUNLOCK(gXitk->display);  */
-      widget_xevent_notify(&myevent);
+      xitk_xevent_notify(&myevent);
       /*
     } else {  
       XUNLOCK(gXitk->display); 
-      xine_usec_sleep(16666); // 1/60 sec
+      xitk_usec_sleep(16666); // 1/60 sec
     } 
 
     */
@@ -661,14 +794,91 @@ void widget_run(void) {
 
  killing_end:
 
-  gui_list_free(gXitk->list);
-  gui_list_free(gXitk->gfx);
-  free(gXitk);
+  xitk_list_free(gXitk->list);
+  xitk_list_free(gXitk->gfx);
+  xitk_config_deinit(gXitk->config);
+  
+  XITK_FREE(gXitk);
 }
 
 /*
  * Stop the wait xevent loop
  */
-void widget_stop(void) {
+void xitk_stop(void) {
   gXitk->running = 0;
 }
+ 
+char *xitk_get_system_font(void) {
+  return xitk_config_get_system_font(gXitk->config);
+}
+char *xitk_get_default_font(void) {
+  return xitk_config_get_default_font(gXitk->config);
+}
+int xitk_get_black_color(void) {
+  return xitk_config_get_black_color(gXitk->config);
+}
+int xitk_get_white_color(void) {
+  return xitk_config_get_white_color(gXitk->config);
+}
+int xitk_get_background_color(void) {
+  return xitk_config_get_background_color(gXitk->config);
+}
+int xitk_get_focus_color(void) {
+  return xitk_config_get_focus_color(gXitk->config);
+}
+int xitk_get_select_color(void) {
+  return xitk_config_get_select_color(gXitk->config);
+}
+
+/*
+ *
+ */
+char *xitk_set_locale(void) {
+  char *cur_locale = NULL;
+  
+  if(setlocale (LC_ALL,"") == NULL) {
+    XITK_WARNING("locale not supported by C library\n");
+    return NULL;
+  }
+  
+  cur_locale = setlocale(LC_ALL, NULL);
+  
+  return cur_locale;
+}
+
+/*
+ * Return home directory.
+ */
+const char *xitk_get_homedir(void) {
+  struct passwd *pw = NULL;
+  char *homedir = NULL;
+#ifdef HAVE_GETPWUID_R
+  int ret;
+  struct passwd pwd;
+  char *buffer = NULL;
+  int bufsize = 128;
+
+  buffer = (char *) xmalloc(bufsize);
+  
+  if((ret = getpwuid_r(getuid(), &pwd, buffer, bufsize, &pw)) < 0) {
+#else
+  if((pw = getpwuid(getuid())) == NULL) {
+#endif
+    if((homedir = getenv("HOME")) == NULL) {
+      XITK_WARNING("Unable to get home directory, set it to /tmp.\n");
+      homedir = strdup("/tmp");
+    }
+  }
+  else {
+    if(pw) 
+      homedir = strdup(pw->pw_dir);
+  }
+  
+  
+#ifdef HAVE_GETPWUID_R
+  if(buffer) 
+    XITK_FREE(buffer);
+#endif
+  
+  return homedir;
+}
